# -*- coding: utf-8 -*-

#
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SignalFrame(object):
    def setupUi(self, SignalFrame):
        SignalFrame.setObjectName("SignalFrame")
        SignalFrame.resize(1057, 652)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SignalFrame.sizePolicy().hasHeightForWidth())
        SignalFrame.setSizePolicy(sizePolicy)
        SignalFrame.setMinimumSize(QtCore.QSize(0, 0))
        SignalFrame.setMaximumSize(QtCore.QSize(16777215, 16777215))
        SignalFrame.setSizeIncrement(QtCore.QSize(0, 0))
        SignalFrame.setBaseSize(QtCore.QSize(0, 0))
        SignalFrame.setMouseTracking(False)
        SignalFrame.setAcceptDrops(True)
        SignalFrame.setAutoFillBackground(False)
        SignalFrame.setStyleSheet("")
        SignalFrame.setFrameShape(QtWidgets.QFrame.NoFrame)
        SignalFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        SignalFrame.setLineWidth(1)
        self.horizontalLayout = QtWidgets.QHBoxLayout(SignalFrame)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.lSignalViewText = QtWidgets.QLabel(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSignalViewText.sizePolicy().hasHeightForWidth())
        self.lSignalViewText.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setUnderline(False)
        self.lSignalViewText.setFont(font)
        self.lSignalViewText.setObjectName("lSignalViewText")
        self.gridLayout_2.addWidget(self.lSignalViewText, 17, 0, 1, 1)
        self.spinBoxBitsPerSymbol = QtWidgets.QSpinBox(SignalFrame)
        self.spinBoxBitsPerSymbol.setMinimum(1)
        self.spinBoxBitsPerSymbol.setMaximum(10)
        self.spinBoxBitsPerSymbol.setObjectName("spinBoxBitsPerSymbol")
        self.gridLayout_2.addWidget(self.spinBoxBitsPerSymbol, 11, 1, 1, 1)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.btnSaveSignal = QtWidgets.QToolButton(SignalFrame)
        self.btnSaveSignal.setMinimumSize(QtCore.QSize(24, 24))
        self.btnSaveSignal.setMaximumSize(QtCore.QSize(24, 24))
        icon = QtGui.QIcon.fromTheme("document-save")
        self.btnSaveSignal.setIcon(icon)
        self.btnSaveSignal.setObjectName("btnSaveSignal")
        self.gridLayout.addWidget(self.btnSaveSignal, 0, 3, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 2, 1, 1)
        self.btnCloseSignal = QtWidgets.QToolButton(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnCloseSignal.sizePolicy().hasHeightForWidth())
        self.btnCloseSignal.setSizePolicy(sizePolicy)
        self.btnCloseSignal.setMinimumSize(QtCore.QSize(24, 24))
        self.btnCloseSignal.setMaximumSize(QtCore.QSize(24, 24))
        self.btnCloseSignal.setStyleSheet("color:red;")
        icon = QtGui.QIcon.fromTheme("window-close")
        self.btnCloseSignal.setIcon(icon)
        self.btnCloseSignal.setObjectName("btnCloseSignal")
        self.gridLayout.addWidget(self.btnCloseSignal, 0, 9, 1, 1)
        self.lSignalTyp = QtWidgets.QLabel(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSignalTyp.sizePolicy().hasHeightForWidth())
        self.lSignalTyp.setSizePolicy(sizePolicy)
        self.lSignalTyp.setObjectName("lSignalTyp")
        self.gridLayout.addWidget(self.lSignalTyp, 0, 1, 1, 1)
        self.lSignalNr = QtWidgets.QLabel(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSignalNr.sizePolicy().hasHeightForWidth())
        self.lSignalNr.setSizePolicy(sizePolicy)
        self.lSignalNr.setWordWrap(False)
        self.lSignalNr.setIndent(-1)
        self.lSignalNr.setObjectName("lSignalNr")
        self.gridLayout.addWidget(self.lSignalNr, 0, 0, 1, 1)
        self.btnInfo = QtWidgets.QToolButton(SignalFrame)
        self.btnInfo.setMinimumSize(QtCore.QSize(24, 24))
        self.btnInfo.setMaximumSize(QtCore.QSize(24, 24))
        icon = QtGui.QIcon.fromTheme("dialog-information")
        self.btnInfo.setIcon(icon)
        self.btnInfo.setObjectName("btnInfo")
        self.gridLayout.addWidget(self.btnInfo, 0, 6, 1, 1)
        self.btnReplay = QtWidgets.QToolButton(SignalFrame)
        self.btnReplay.setMinimumSize(QtCore.QSize(24, 24))
        self.btnReplay.setMaximumSize(QtCore.QSize(24, 24))
        self.btnReplay.setText("")
        icon = QtGui.QIcon.fromTheme("media-playback-start")
        self.btnReplay.setIcon(icon)
        self.btnReplay.setObjectName("btnReplay")
        self.gridLayout.addWidget(self.btnReplay, 0, 5, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 2)
        self.cbProtoView = QtWidgets.QComboBox(SignalFrame)
        self.cbProtoView.setObjectName("cbProtoView")
        self.cbProtoView.addItem("")
        self.cbProtoView.addItem("")
        self.cbProtoView.addItem("")
        self.gridLayout_2.addWidget(self.cbProtoView, 23, 1, 1, 1)
        self.lCenterSpacing = QtWidgets.QLabel(SignalFrame)
        self.lCenterSpacing.setObjectName("lCenterSpacing")
        self.gridLayout_2.addWidget(self.lCenterSpacing, 4, 0, 1, 1)
        self.spinBoxTolerance = QtWidgets.QSpinBox(SignalFrame)
        self.spinBoxTolerance.setMinimumSize(QtCore.QSize(100, 0))
        self.spinBoxTolerance.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.spinBoxTolerance.setMaximum(9999)
        self.spinBoxTolerance.setObjectName("spinBoxTolerance")
        self.gridLayout_2.addWidget(self.spinBoxTolerance, 8, 1, 1, 1)
        self.lSamplesPerSymbol = QtWidgets.QLabel(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSamplesPerSymbol.sizePolicy().hasHeightForWidth())
        self.lSamplesPerSymbol.setSizePolicy(sizePolicy)
        self.lSamplesPerSymbol.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse)
        self.lSamplesPerSymbol.setObjectName("lSamplesPerSymbol")
        self.gridLayout_2.addWidget(self.lSamplesPerSymbol, 5, 0, 1, 1)
        self.lErrorTolerance = QtWidgets.QLabel(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lErrorTolerance.sizePolicy().hasHeightForWidth())
        self.lErrorTolerance.setSizePolicy(sizePolicy)
        self.lErrorTolerance.setMinimumSize(QtCore.QSize(0, 0))
        self.lErrorTolerance.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lErrorTolerance.setObjectName("lErrorTolerance")
        self.gridLayout_2.addWidget(self.lErrorTolerance, 8, 0, 1, 1)
        self.lCenterOffset = QtWidgets.QLabel(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lCenterOffset.sizePolicy().hasHeightForWidth())
        self.lCenterOffset.setSizePolicy(sizePolicy)
        self.lCenterOffset.setMinimumSize(QtCore.QSize(0, 0))
        self.lCenterOffset.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lCenterOffset.setWhatsThis("")
        self.lCenterOffset.setObjectName("lCenterOffset")
        self.gridLayout_2.addWidget(self.lCenterOffset, 3, 0, 1, 1)
        self.labelModulation = QtWidgets.QLabel(SignalFrame)
        self.labelModulation.setObjectName("labelModulation")
        self.gridLayout_2.addWidget(self.labelModulation, 10, 0, 1, 1)
        self.cbSignalView = QtWidgets.QComboBox(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cbSignalView.sizePolicy().hasHeightForWidth())
        self.cbSignalView.setSizePolicy(sizePolicy)
        self.cbSignalView.setObjectName("cbSignalView")
        self.cbSignalView.addItem("")
        self.cbSignalView.addItem("")
        self.cbSignalView.addItem("")
        self.cbSignalView.addItem("")
        self.gridLayout_2.addWidget(self.cbSignalView, 17, 1, 1, 1)
        self.spinBoxNoiseTreshold = QtWidgets.QDoubleSpinBox(SignalFrame)
        self.spinBoxNoiseTreshold.setSuffix("")
        self.spinBoxNoiseTreshold.setDecimals(4)
        self.spinBoxNoiseTreshold.setMaximum(1.0)
        self.spinBoxNoiseTreshold.setSingleStep(0.0001)
        self.spinBoxNoiseTreshold.setObjectName("spinBoxNoiseTreshold")
        self.gridLayout_2.addWidget(self.spinBoxNoiseTreshold, 2, 1, 1, 1)
        self.spinBoxCenterSpacing = QtWidgets.QDoubleSpinBox(SignalFrame)
        self.spinBoxCenterSpacing.setDecimals(4)
        self.spinBoxCenterSpacing.setMinimum(0.0001)
        self.spinBoxCenterSpacing.setMaximum(5.0)
        self.spinBoxCenterSpacing.setSingleStep(0.0001)
        self.spinBoxCenterSpacing.setProperty("value", 1.0)
        self.spinBoxCenterSpacing.setObjectName("spinBoxCenterSpacing")
        self.gridLayout_2.addWidget(self.spinBoxCenterSpacing, 4, 1, 1, 1)
        self.lineEditSignalName = QtWidgets.QLineEdit(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditSignalName.sizePolicy().hasHeightForWidth())
        self.lineEditSignalName.setSizePolicy(sizePolicy)
        self.lineEditSignalName.setMinimumSize(QtCore.QSize(214, 0))
        self.lineEditSignalName.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lineEditSignalName.setAcceptDrops(False)
        self.lineEditSignalName.setObjectName("lineEditSignalName")
        self.gridLayout_2.addWidget(self.lineEditSignalName, 1, 0, 1, 2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setSpacing(7)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.cbModulationType = QtWidgets.QComboBox(SignalFrame)
        self.cbModulationType.setObjectName("cbModulationType")
        self.cbModulationType.addItem("")
        self.cbModulationType.addItem("")
        self.cbModulationType.addItem("")
        self.horizontalLayout_5.addWidget(self.cbModulationType)
        self.btnAdvancedModulationSettings = QtWidgets.QToolButton(SignalFrame)
        icon = QtGui.QIcon.fromTheme("configure")
        self.btnAdvancedModulationSettings.setIcon(icon)
        self.btnAdvancedModulationSettings.setIconSize(QtCore.QSize(16, 16))
        self.btnAdvancedModulationSettings.setObjectName("btnAdvancedModulationSettings")
        self.horizontalLayout_5.addWidget(self.btnAdvancedModulationSettings)
        self.gridLayout_2.addLayout(self.horizontalLayout_5, 10, 1, 1, 1)
        self.chkBoxSyncSelection = QtWidgets.QCheckBox(SignalFrame)
        self.chkBoxSyncSelection.setChecked(True)
        self.chkBoxSyncSelection.setObjectName("chkBoxSyncSelection")
        self.gridLayout_2.addWidget(self.chkBoxSyncSelection, 24, 0, 1, 1)
        self.labelSpectrogramMin = QtWidgets.QLabel(SignalFrame)
        self.labelSpectrogramMin.setObjectName("labelSpectrogramMin")
        self.gridLayout_2.addWidget(self.labelSpectrogramMin, 21, 0, 1, 1)
        self.labelSpectrogramMax = QtWidgets.QLabel(SignalFrame)
        self.labelSpectrogramMax.setObjectName("labelSpectrogramMax")
        self.gridLayout_2.addWidget(self.labelSpectrogramMax, 22, 0, 1, 1)
        self.chkBoxShowProtocol = QtWidgets.QCheckBox(SignalFrame)
        self.chkBoxShowProtocol.setObjectName("chkBoxShowProtocol")
        self.gridLayout_2.addWidget(self.chkBoxShowProtocol, 23, 0, 1, 1)
        self.spinBoxSamplesPerSymbol = QtWidgets.QSpinBox(SignalFrame)
        self.spinBoxSamplesPerSymbol.setMinimumSize(QtCore.QSize(100, 0))
        self.spinBoxSamplesPerSymbol.setMinimum(1)
        self.spinBoxSamplesPerSymbol.setMaximum(999999999)
        self.spinBoxSamplesPerSymbol.setObjectName("spinBoxSamplesPerSymbol")
        self.gridLayout_2.addWidget(self.spinBoxSamplesPerSymbol, 5, 1, 1, 1)
        self.btnAutoDetect = QtWidgets.QToolButton(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAutoDetect.sizePolicy().hasHeightForWidth())
        self.btnAutoDetect.setSizePolicy(sizePolicy)
        self.btnAutoDetect.setIconSize(QtCore.QSize(16, 16))
        self.btnAutoDetect.setCheckable(False)
        self.btnAutoDetect.setChecked(False)
        self.btnAutoDetect.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
        self.btnAutoDetect.setArrowType(QtCore.Qt.NoArrow)
        self.btnAutoDetect.setObjectName("btnAutoDetect")
        self.gridLayout_2.addWidget(self.btnAutoDetect, 13, 0, 1, 2)
        self.line = QtWidgets.QFrame(SignalFrame)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_2.addWidget(self.line, 15, 0, 1, 2)
        self.sliderSpectrogramMin = QtWidgets.QSlider(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sliderSpectrogramMin.sizePolicy().hasHeightForWidth())
        self.sliderSpectrogramMin.setSizePolicy(sizePolicy)
        self.sliderSpectrogramMin.setMinimum(-150)
        self.sliderSpectrogramMin.setMaximum(10)
        self.sliderSpectrogramMin.setOrientation(QtCore.Qt.Horizontal)
        self.sliderSpectrogramMin.setObjectName("sliderSpectrogramMin")
        self.gridLayout_2.addWidget(self.sliderSpectrogramMin, 21, 1, 1, 1)
        self.labelFFTWindowSize = QtWidgets.QLabel(SignalFrame)
        self.labelFFTWindowSize.setObjectName("labelFFTWindowSize")
        self.gridLayout_2.addWidget(self.labelFFTWindowSize, 20, 0, 1, 1)
        self.sliderSpectrogramMax = QtWidgets.QSlider(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sliderSpectrogramMax.sizePolicy().hasHeightForWidth())
        self.sliderSpectrogramMax.setSizePolicy(sizePolicy)
        self.sliderSpectrogramMax.setMinimum(-150)
        self.sliderSpectrogramMax.setMaximum(10)
        self.sliderSpectrogramMax.setOrientation(QtCore.Qt.Horizontal)
        self.sliderSpectrogramMax.setObjectName("sliderSpectrogramMax")
        self.gridLayout_2.addWidget(self.sliderSpectrogramMax, 22, 1, 1, 1)
        self.spinBoxCenterOffset = QtWidgets.QDoubleSpinBox(SignalFrame)
        self.spinBoxCenterOffset.setMinimumSize(QtCore.QSize(100, 0))
        self.spinBoxCenterOffset.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.spinBoxCenterOffset.setDecimals(4)
        self.spinBoxCenterOffset.setMinimum(-3.15)
        self.spinBoxCenterOffset.setMaximum(6.28)
        self.spinBoxCenterOffset.setSingleStep(0.0001)
        self.spinBoxCenterOffset.setObjectName("spinBoxCenterOffset")
        self.gridLayout_2.addWidget(self.spinBoxCenterOffset, 3, 1, 1, 1)
        self.labelNoise = QtWidgets.QLabel(SignalFrame)
        self.labelNoise.setObjectName("labelNoise")
        self.gridLayout_2.addWidget(self.labelNoise, 2, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem1, 14, 0, 1, 1)
        self.sliderFFTWindowSize = QtWidgets.QSlider(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sliderFFTWindowSize.sizePolicy().hasHeightForWidth())
        self.sliderFFTWindowSize.setSizePolicy(sizePolicy)
        self.sliderFFTWindowSize.setMinimum(6)
        self.sliderFFTWindowSize.setMaximum(15)
        self.sliderFFTWindowSize.setOrientation(QtCore.Qt.Horizontal)
        self.sliderFFTWindowSize.setObjectName("sliderFFTWindowSize")
        self.gridLayout_2.addWidget(self.sliderFFTWindowSize, 20, 1, 1, 1)
        self.lBitsPerSymbol = QtWidgets.QLabel(SignalFrame)
        self.lBitsPerSymbol.setObjectName("lBitsPerSymbol")
        self.gridLayout_2.addWidget(self.lBitsPerSymbol, 11, 0, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout_2)
        self.splitter = QtWidgets.QSplitter(SignalFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy)
        self.splitter.setStyleSheet("QSplitter::handle:vertical {\n"
"margin: 4px 0px;\n"
"    background-color: qlineargradient(x1:0, y1:0, x2:1, y2:0, \n"
"stop:0 rgba(255, 255, 255, 0), \n"
"stop:0.5 rgba(100, 100, 100, 100), \n"
"stop:1 rgba(255, 255, 255, 0));\n"
"    image: url(:/icons/icons/splitter_handle_horizontal.svg);\n"
"}")
        self.splitter.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.splitter.setLineWidth(1)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setHandleWidth(6)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.stackedWidget = QtWidgets.QStackedWidget(self.layoutWidget)
        self.stackedWidget.setLineWidth(0)
        self.stackedWidget.setObjectName("stackedWidget")
        self.pageSignal = QtWidgets.QWidget()
        self.pageSignal.setObjectName("pageSignal")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.pageSignal)
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_6.setSpacing(0)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.gvSignal = EpicGraphicView(self.pageSignal)
        self.gvSignal.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.gvSignal.sizePolicy().hasHeightForWidth())
        self.gvSignal.setSizePolicy(sizePolicy)
        self.gvSignal.setMinimumSize(QtCore.QSize(0, 150))
        self.gvSignal.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.gvSignal.setMouseTracking(True)
        self.gvSignal.setFocusPolicy(QtCore.Qt.WheelFocus)
        self.gvSignal.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.gvSignal.setAutoFillBackground(False)
        self.gvSignal.setStyleSheet("")
        self.gvSignal.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.gvSignal.setFrameShadow(QtWidgets.QFrame.Raised)
        self.gvSignal.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.gvSignal.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.gvSignal.setInteractive(False)
        self.gvSignal.setRenderHints(QtGui.QPainter.Antialiasing|QtGui.QPainter.TextAntialiasing)
        self.gvSignal.setDragMode(QtWidgets.QGraphicsView.NoDrag)
        self.gvSignal.setCacheMode(QtWidgets.QGraphicsView.CacheNone)
        self.gvSignal.setTransformationAnchor(QtWidgets.QGraphicsView.NoAnchor)
        self.gvSignal.setResizeAnchor(QtWidgets.QGraphicsView.NoAnchor)
        self.gvSignal.setViewportUpdateMode(QtWidgets.QGraphicsView.MinimalViewportUpdate)
        self.gvSignal.setRubberBandSelectionMode(QtCore.Qt.ContainsItemShape)
        self.gvSignal.setOptimizationFlags(QtWidgets.QGraphicsView.DontClipPainter|QtWidgets.QGraphicsView.DontSavePainterState)
        self.gvSignal.setObjectName("gvSignal")
        self.horizontalLayout_6.addWidget(self.gvSignal)
        self.stackedWidget.addWidget(self.pageSignal)
        self.pageSpectrogram = QtWidgets.QWidget()
        self.pageSpectrogram.setObjectName("pageSpectrogram")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.pageSpectrogram)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_4.setSpacing(0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.gvSpectrogram = SpectrogramGraphicView(self.pageSpectrogram)
        self.gvSpectrogram.setMouseTracking(True)
        self.gvSpectrogram.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.gvSpectrogram.setInteractive(False)
        self.gvSpectrogram.setRenderHints(QtGui.QPainter.TextAntialiasing)
        self.gvSpectrogram.setCacheMode(QtWidgets.QGraphicsView.CacheNone)
        self.gvSpectrogram.setTransformationAnchor(QtWidgets.QGraphicsView.NoAnchor)
        self.gvSpectrogram.setViewportUpdateMode(QtWidgets.QGraphicsView.MinimalViewportUpdate)
        self.gvSpectrogram.setOptimizationFlags(QtWidgets.QGraphicsView.DontClipPainter|QtWidgets.QGraphicsView.DontSavePainterState)
        self.gvSpectrogram.setObjectName("gvSpectrogram")
        self.horizontalLayout_4.addWidget(self.gvSpectrogram)
        self.stackedWidget.addWidget(self.pageSpectrogram)
        self.pageLoading = QtWidgets.QWidget()
        self.pageLoading.setObjectName("pageLoading")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.pageLoading)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.labelLoadingAutoInterpretation = QtWidgets.QLabel(self.pageLoading)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.labelLoadingAutoInterpretation.setFont(font)
        self.labelLoadingAutoInterpretation.setWordWrap(True)
        self.labelLoadingAutoInterpretation.setObjectName("labelLoadingAutoInterpretation")
        self.verticalLayout_2.addWidget(self.labelLoadingAutoInterpretation)
        self.stackedWidget.addWidget(self.pageLoading)
        self.horizontalLayout_2.addWidget(self.stackedWidget)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.lYScale = QtWidgets.QLabel(self.layoutWidget)
        self.lYScale.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.lYScale.setObjectName("lYScale")
        self.verticalLayout_5.addWidget(self.lYScale)
        self.sliderYScale = QtWidgets.QSlider(self.layoutWidget)
        self.sliderYScale.setMinimum(1)
        self.sliderYScale.setMaximum(100)
        self.sliderYScale.setOrientation(QtCore.Qt.Vertical)
        self.sliderYScale.setTickPosition(QtWidgets.QSlider.TicksBelow)
        self.sliderYScale.setObjectName("sliderYScale")
        self.verticalLayout_5.addWidget(self.sliderYScale)
        self.horizontalLayout_2.addLayout(self.verticalLayout_5)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.btnShowHideStartEnd = QtWidgets.QToolButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnShowHideStartEnd.sizePolicy().hasHeightForWidth())
        self.btnShowHideStartEnd.setSizePolicy(sizePolicy)
        self.btnShowHideStartEnd.setAutoFillBackground(False)
        self.btnShowHideStartEnd.setStyleSheet("")
        icon = QtGui.QIcon.fromTheme("arrow-down-double")
        self.btnShowHideStartEnd.setIcon(icon)
        self.btnShowHideStartEnd.setCheckable(True)
        self.btnShowHideStartEnd.setObjectName("btnShowHideStartEnd")
        self.horizontalLayout_3.addWidget(self.btnShowHideStartEnd)
        self.lNumSelectedSamples = QtWidgets.QLabel(self.layoutWidget)
        self.lNumSelectedSamples.setObjectName("lNumSelectedSamples")
        self.horizontalLayout_3.addWidget(self.lNumSelectedSamples)
        self.lTextSelectedSamples = QtWidgets.QLabel(self.layoutWidget)
        self.lTextSelectedSamples.setObjectName("lTextSelectedSamples")
        self.horizontalLayout_3.addWidget(self.lTextSelectedSamples)
        self.line_3 = QtWidgets.QFrame(self.layoutWidget)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout_3.addWidget(self.line_3)
        self.lDuration = QtWidgets.QLabel(self.layoutWidget)
        self.lDuration.setToolTip("")
        self.lDuration.setObjectName("lDuration")
        self.horizontalLayout_3.addWidget(self.lDuration)
        self.line_2 = QtWidgets.QFrame(self.layoutWidget)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_3.addWidget(self.line_2)
        self.labelRSSI = QtWidgets.QLabel(self.layoutWidget)
        self.labelRSSI.setObjectName("labelRSSI")
        self.horizontalLayout_3.addWidget(self.labelRSSI)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.btnFilter = QtWidgets.QToolButton(self.layoutWidget)
        icon = QtGui.QIcon.fromTheme("view-filter")
        self.btnFilter.setIcon(icon)
        self.btnFilter.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
        self.btnFilter.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.btnFilter.setArrowType(QtCore.Qt.NoArrow)
        self.btnFilter.setObjectName("btnFilter")
        self.horizontalLayout_3.addWidget(self.btnFilter)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.additionalInfos = QtWidgets.QHBoxLayout()
        self.additionalInfos.setSpacing(6)
        self.additionalInfos.setObjectName("additionalInfos")
        self.lStart = QtWidgets.QLabel(self.layoutWidget)
        self.lStart.setObjectName("lStart")
        self.additionalInfos.addWidget(self.lStart)
        self.spinBoxSelectionStart = QtWidgets.QSpinBox(self.layoutWidget)
        self.spinBoxSelectionStart.setReadOnly(False)
        self.spinBoxSelectionStart.setMaximum(99999999)
        self.spinBoxSelectionStart.setObjectName("spinBoxSelectionStart")
        self.additionalInfos.addWidget(self.spinBoxSelectionStart)
        self.lEnd = QtWidgets.QLabel(self.layoutWidget)
        self.lEnd.setObjectName("lEnd")
        self.additionalInfos.addWidget(self.lEnd)
        self.spinBoxSelectionEnd = QtWidgets.QSpinBox(self.layoutWidget)
        self.spinBoxSelectionEnd.setMaximum(99999999)
        self.spinBoxSelectionEnd.setObjectName("spinBoxSelectionEnd")
        self.additionalInfos.addWidget(self.spinBoxSelectionEnd)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.additionalInfos.addItem(spacerItem3)
        self.lZoomText = QtWidgets.QLabel(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lZoomText.sizePolicy().hasHeightForWidth())
        self.lZoomText.setSizePolicy(sizePolicy)
        self.lZoomText.setMinimumSize(QtCore.QSize(0, 0))
        self.lZoomText.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setItalic(False)
        font.setUnderline(False)
        self.lZoomText.setFont(font)
        self.lZoomText.setTextFormat(QtCore.Qt.PlainText)
        self.lZoomText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.lZoomText.setObjectName("lZoomText")
        self.additionalInfos.addWidget(self.lZoomText)
        self.spinBoxXZoom = QtWidgets.QSpinBox(self.layoutWidget)
        self.spinBoxXZoom.setMinimum(100)
        self.spinBoxXZoom.setMaximum(999999999)
        self.spinBoxXZoom.setObjectName("spinBoxXZoom")
        self.additionalInfos.addWidget(self.spinBoxXZoom)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.additionalInfos.addItem(spacerItem4)
        self.lSamplesInView = QtWidgets.QLabel(self.layoutWidget)
        self.lSamplesInView.setObjectName("lSamplesInView")
        self.additionalInfos.addWidget(self.lSamplesInView)
        self.lStrich = QtWidgets.QLabel(self.layoutWidget)
        self.lStrich.setObjectName("lStrich")
        self.additionalInfos.addWidget(self.lStrich)
        self.lSamplesTotal = QtWidgets.QLabel(self.layoutWidget)
        self.lSamplesTotal.setObjectName("lSamplesTotal")
        self.additionalInfos.addWidget(self.lSamplesTotal)
        self.lSamplesViewText = QtWidgets.QLabel(self.layoutWidget)
        self.lSamplesViewText.setObjectName("lSamplesViewText")
        self.additionalInfos.addWidget(self.lSamplesViewText)
        self.verticalLayout.addLayout(self.additionalInfos)
        self.txtEdProto = TextEditProtocolView(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtEdProto.sizePolicy().hasHeightForWidth())
        self.txtEdProto.setSizePolicy(sizePolicy)
        self.txtEdProto.setMinimumSize(QtCore.QSize(0, 80))
        self.txtEdProto.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.txtEdProto.setBaseSize(QtCore.QSize(0, 0))
        self.txtEdProto.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.txtEdProto.setAcceptDrops(False)
        self.txtEdProto.setObjectName("txtEdProto")
        self.horizontalLayout.addWidget(self.splitter)

        self.retranslateUi(SignalFrame)
        self.stackedWidget.setCurrentIndex(0)
        SignalFrame.setTabOrder(self.btnSaveSignal, self.btnReplay)
        SignalFrame.setTabOrder(self.btnReplay, self.btnInfo)
        SignalFrame.setTabOrder(self.btnInfo, self.btnCloseSignal)
        SignalFrame.setTabOrder(self.btnCloseSignal, self.gvSignal)
        SignalFrame.setTabOrder(self.gvSignal, self.lineEditSignalName)
        SignalFrame.setTabOrder(self.lineEditSignalName, self.spinBoxNoiseTreshold)
        SignalFrame.setTabOrder(self.spinBoxNoiseTreshold, self.spinBoxCenterOffset)
        SignalFrame.setTabOrder(self.spinBoxCenterOffset, self.spinBoxCenterSpacing)
        SignalFrame.setTabOrder(self.spinBoxCenterSpacing, self.spinBoxSamplesPerSymbol)
        SignalFrame.setTabOrder(self.spinBoxSamplesPerSymbol, self.spinBoxTolerance)
        SignalFrame.setTabOrder(self.spinBoxTolerance, self.cbModulationType)
        SignalFrame.setTabOrder(self.cbModulationType, self.spinBoxBitsPerSymbol)
        SignalFrame.setTabOrder(self.spinBoxBitsPerSymbol, self.btnAdvancedModulationSettings)
        SignalFrame.setTabOrder(self.btnAdvancedModulationSettings, self.btnShowHideStartEnd)
        SignalFrame.setTabOrder(self.btnShowHideStartEnd, self.btnAutoDetect)
        SignalFrame.setTabOrder(self.btnAutoDetect, self.txtEdProto)
        SignalFrame.setTabOrder(self.txtEdProto, self.cbSignalView)
        SignalFrame.setTabOrder(self.cbSignalView, self.sliderFFTWindowSize)
        SignalFrame.setTabOrder(self.sliderFFTWindowSize, self.sliderSpectrogramMin)
        SignalFrame.setTabOrder(self.sliderSpectrogramMin, self.sliderSpectrogramMax)
        SignalFrame.setTabOrder(self.sliderSpectrogramMax, self.cbProtoView)
        SignalFrame.setTabOrder(self.cbProtoView, self.chkBoxShowProtocol)
        SignalFrame.setTabOrder(self.chkBoxShowProtocol, self.chkBoxSyncSelection)
        SignalFrame.setTabOrder(self.chkBoxSyncSelection, self.sliderYScale)
        SignalFrame.setTabOrder(self.sliderYScale, self.btnFilter)
        SignalFrame.setTabOrder(self.btnFilter, self.spinBoxSelectionStart)
        SignalFrame.setTabOrder(self.spinBoxSelectionStart, self.spinBoxSelectionEnd)
        SignalFrame.setTabOrder(self.spinBoxSelectionEnd, self.spinBoxXZoom)
        SignalFrame.setTabOrder(self.spinBoxXZoom, self.gvSpectrogram)

    def retranslateUi(self, SignalFrame):
        _translate = QtCore.QCoreApplication.translate
        SignalFrame.setWindowTitle(_translate("SignalFrame", "Frame"))
        self.lSignalViewText.setText(_translate("SignalFrame", "Signal view:"))
        self.spinBoxBitsPerSymbol.setToolTip(_translate("SignalFrame", "<html><head/><body><p><span style=\" font-weight:600;\">Higher order</span> modulations can carry <span style=\" font-weight:600;\">multiple</span> bits with <span style=\" font-weight:600;\">each</span> symbol. Configure <span style=\" font-weight:600;\">how many</span> bits are represented by a symbol. (Default = Binary modulation with one bit per symbol)</p></body></html>"))
        self.btnSaveSignal.setText(_translate("SignalFrame", "..."))
        self.btnCloseSignal.setText(_translate("SignalFrame", "X"))
        self.lSignalTyp.setText(_translate("SignalFrame", "<Signaltyp>"))
        self.lSignalNr.setText(_translate("SignalFrame", "1:"))
        self.btnInfo.setText(_translate("SignalFrame", "..."))
        self.btnReplay.setToolTip(_translate("SignalFrame", "Replay signal"))
        self.cbProtoView.setItemText(0, _translate("SignalFrame", "Bits"))
        self.cbProtoView.setItemText(1, _translate("SignalFrame", "Hex"))
        self.cbProtoView.setItemText(2, _translate("SignalFrame", "ASCII"))
        self.lCenterSpacing.setToolTip(_translate("SignalFrame", "<html><head/><body><p>For <span style=\" font-weight:600;\">higher order</span> modulations (&gt; 1 Bits/Symbol), there are <span style=\" font-weight:600;\">multiple</span> centers. We assume that the <span style=\" font-weight:600;\">spacing</span> between all possible symbols is <span style=\" font-weight:600;\">constant</span>. Therefore you configure the spacing between centers.</p></body></html>"))
        self.lCenterSpacing.setText(_translate("SignalFrame", "Center spacing:"))
        self.spinBoxTolerance.setToolTip(_translate("SignalFrame", "<html><head/><body><p>This is the error tolerance for determining the <span style=\" font-weight:600;\">pulse lengths</span> in the demodulated signal.</p><p><span style=\" font-weight:400; font-style:italic;\">Example:</span> Say, we are reading a ones pulse and the tolerance value was set to 5. Then 5 errors (which must follow sequentially) are accepted.</p><p>Tune this value if you have <span style=\" font-weight:600;\">spiky data</span> after demodulation.</p></body></html>"))
        self.lSamplesPerSymbol.setToolTip(_translate("SignalFrame", "<html><head/><body><p>This is the length of one symbol <span style=\" font-weight:600;\">in samples</span>. For <span style=\" font-weight:600;\">binary modulations </span>(default) this is the <span style=\" font-weight:600;\">bit length</span>.</p></body></html>"))
        self.lSamplesPerSymbol.setText(_translate("SignalFrame", "Samples/Symbol:"))
        self.lErrorTolerance.setToolTip(_translate("SignalFrame", "<html><head/><body><p>This is the error tolerance for determining the <span style=\" font-weight:600;\">pulse lengths</span> in the demodulated signal.</p><p><span style=\" font-weight:400; font-style:italic;\">Example:</span> Say, we are reading a ones pulse and the tolerance value was set to 5. Then 5 errors (which must follow sequentially) are accepted.</p><p>Tune this value if you have <span style=\" font-weight:600;\">spiky data</span> after demodulation.</p></body></html>"))
        self.lErrorTolerance.setText(_translate("SignalFrame", "Error tolerance:"))
        self.lCenterOffset.setToolTip(_translate("SignalFrame", "<html><head/><body><p>This is the threshold used for determining if a <span style=\" font-weight:600;\">bit is one or zero</span>. You can set it here or grab the middle of the area in <span style=\" font-style:italic;\">Quadrature Demod View.</span></p></body></html>"))
        self.lCenterOffset.setText(_translate("SignalFrame", "Center:"))
        self.labelModulation.setToolTip(_translate("SignalFrame", "<html><head/><body><p>Choose signals modulation:</p><ul><li>Amplitude Shift Keying (ASK)</li><li>Frequency Shift Keying (FSK)</li><li>Phase Shift Keying (PSK)</li></ul></body></html>"))
        self.labelModulation.setText(_translate("SignalFrame", "Modulation:"))
        self.cbSignalView.setToolTip(_translate("SignalFrame", "<html><head/><body><p>Choose the view of your signal. Analog, Demodulated, Spectrogram or I/Q view.</p><p>The quadrature demodulation uses a <span style=\" font-weight:600;\">threshold of magnitudes,</span> to <span style=\" font-weight:600;\">suppress noise</span>. All samples with a magnitude lower than this threshold will be eliminated after demodulation.</p><p>Tune this value by selecting a <span style=\" font-style:italic;\">noisy area</span> and mark it as noise using <span style=\" font-weight:600;\">context menu</span>.</p><p><span style=\" font-weight:600;\">Signal colors</span>:<br/>I =&gt; <span style=\" font-weight:600; color:#3232e1;\">blue</span><br/>Q =&gt; <span style=\" font-weight:600;\">black</span></p></body></html>"))
        self.cbSignalView.setItemText(0, _translate("SignalFrame", "Analog"))
        self.cbSignalView.setItemText(1, _translate("SignalFrame", "Demodulated"))
        self.cbSignalView.setItemText(2, _translate("SignalFrame", "Spectrogram"))
        self.cbSignalView.setItemText(3, _translate("SignalFrame", "I/Q view"))
        self.spinBoxNoiseTreshold.setToolTip(_translate("SignalFrame", "<html><head/><body><p>Set the <span style=\" font-weight:600;\">noise magnitude</span> of your signal. You can tune this value to mute noise in your signal and reveal the true data.</p></body></html>"))
        self.spinBoxCenterSpacing.setToolTip(_translate("SignalFrame", "<html><head/><body><p>For <span style=\" font-weight:600;\">higher order</span> modulations (&gt; 1 Bits/Symbol), there are <span style=\" font-weight:600;\">multiple</span> centers. We assume that the <span style=\" font-weight:600;\">spacing</span> between all possible symbols is <span style=\" font-weight:600;\">constant</span>. Therefore you configure the spacing between centers.</p></body></html>"))
        self.lineEditSignalName.setText(_translate("SignalFrame", "SignalName"))
        self.cbModulationType.setToolTip(_translate("SignalFrame", "<html><head/><body><p>Choose signals modulation:</p><ul><li>Amplitude Shift Keying (ASK)</li><li>Frequency Shift Keying (FSK)</li><li>Phase Shift Keying (PSK)</li></ul></body></html>"))
        self.cbModulationType.setItemText(0, _translate("SignalFrame", "ASK"))
        self.cbModulationType.setItemText(1, _translate("SignalFrame", "FSK"))
        self.cbModulationType.setItemText(2, _translate("SignalFrame", "PSK"))
        self.btnAdvancedModulationSettings.setText(_translate("SignalFrame", "..."))
        self.chkBoxSyncSelection.setToolTip(_translate("SignalFrame", "If this is set to true, your selected protocol bits will show up in the signal view, and vice versa."))
        self.chkBoxSyncSelection.setText(_translate("SignalFrame", "Sync selection"))
        self.labelSpectrogramMin.setText(_translate("SignalFrame", "Data<sub>min</sub>:"))
        self.labelSpectrogramMax.setText(_translate("SignalFrame", "Data<sub>max</sub>:"))
        self.chkBoxShowProtocol.setToolTip(_translate("SignalFrame", "Show the extracted protocol based on the parameters InfoLen, PauseLen and ZeroTreshold (in QuadratureDemod-View).\n"
"\n"
"If you want your protocol to be better separated, edit the PauseLen using right-click menu from a selection in SignalView or ProtocolView."))
        self.chkBoxShowProtocol.setText(_translate("SignalFrame", "Show data as"))
        self.spinBoxSamplesPerSymbol.setToolTip(_translate("SignalFrame", "<html><head/><body><p>This is the length of one symbol <span style=\" font-weight:600;\">in samples</span>. For <span style=\" font-weight:600;\">binary modulations </span>(default) this is the <span style=\" font-weight:600;\">bit length</span>.</p></body></html>"))
        self.btnAutoDetect.setToolTip(_translate("SignalFrame", "<html><head/><body><p>Automatically detect <span style=\" font-weight:600;\">center</span>, <span style=\" font-weight:600;\">bit length</span> and <span style=\" font-weight:600;\">tolerance</span>. You can also choose to additionally detect the <span style=\" font-weight:600;\">noise</span> and <span style=\" font-weight:600;\">modulation</span> when clicking this button.</p></body></html>"))
        self.btnAutoDetect.setText(_translate("SignalFrame", "Autodetect parameters"))
        self.labelFFTWindowSize.setText(_translate("SignalFrame", "FFT window size:"))
        self.spinBoxCenterOffset.setToolTip(_translate("SignalFrame", "<html><head/><body><p>This is the threshold used for determining if a <span style=\" font-weight:600;\">bit is one or zero</span>. You can set it here or grab the middle of the area in <span style=\" font-style:italic;\">Quadrature Demod View</span>.</p></body></html>"))
        self.labelNoise.setToolTip(_translate("SignalFrame", "<html><head/><body><p>Set the <span style=\" font-weight:600;\">noise magnitude</span> of your signal. You can tune this value to mute noise in your signal and reveal the true data.</p></body></html>"))
        self.labelNoise.setText(_translate("SignalFrame", "Noise:"))
        self.lBitsPerSymbol.setToolTip(_translate("SignalFrame", "<html><head/><body><p><span style=\" font-weight:600;\">Higher order</span> modulations can carry <span style=\" font-weight:600;\">multiple</span> bits with <span style=\" font-weight:600;\">each</span> symbol. Configure <span style=\" font-weight:600;\">how many</span> bits are represented by a symbol. (Default = Binary modulation with one bit per symbol)</p></body></html>"))
        self.lBitsPerSymbol.setText(_translate("SignalFrame", "Bits/Symbol:"))
        self.labelLoadingAutoInterpretation.setText(_translate("SignalFrame", "<html><head/><body><p>Running automatic detecting of demodulation parameters.</p><p>You can disable this behaviour for newly loaded signals by unchecking <span style=\" font-weight:600;\">Edit</span> -&gt; <span style=\" font-weight:600;\">Auto detect signals on loading</span>.</p></body></html>"))
        self.lYScale.setText(_translate("SignalFrame", "Y-Scale"))
        self.btnShowHideStartEnd.setText(_translate("SignalFrame", "-"))
        self.lNumSelectedSamples.setToolTip(_translate("SignalFrame", "Number of currently selected samples."))
        self.lNumSelectedSamples.setText(_translate("SignalFrame", "0"))
        self.lTextSelectedSamples.setToolTip(_translate("SignalFrame", "Number of currently selected samples."))
        self.lTextSelectedSamples.setText(_translate("SignalFrame", "selected"))
        self.lDuration.setText(_translate("SignalFrame", "42 µs"))
        self.labelRSSI.setToolTip(_translate("SignalFrame", "<html><head/><body><p>This is the average signal power of the selection. The closer this value is to zero, the stronger the selected signal is.</p></body></html>"))
        self.labelRSSI.setText(_translate("SignalFrame", "0,434 dBm"))
        self.btnFilter.setText(_translate("SignalFrame", "Filter (moving average)"))
        self.lStart.setText(_translate("SignalFrame", "Start:"))
        self.lEnd.setText(_translate("SignalFrame", "End:"))
        self.lZoomText.setToolTip(_translate("SignalFrame", "<html><head/><body><p>Current (relative) Zoom. Standard is 100%, if you zoom in, this factor increases. You can directly set a value in the spinbox or use the <span style=\" font-weight:600;\">mousewheel to zoom</span>.</p></body></html>"))
        self.lZoomText.setText(_translate("SignalFrame", "X-Zoom:"))
        self.spinBoxXZoom.setToolTip(_translate("SignalFrame", "<html><head/><body><p>Current (relative) Zoom. Standard is 100%, if you zoom in, this factor increases. You can directly set a value in the spinbox or use the <span style=\" font-weight:600;\">mousewheel to zoom</span>.</p></body></html>"))
        self.spinBoxXZoom.setSuffix(_translate("SignalFrame", "%"))
        self.lSamplesInView.setText(_translate("SignalFrame", "0"))
        self.lStrich.setText(_translate("SignalFrame", "/"))
        self.lSamplesTotal.setText(_translate("SignalFrame", "0"))
        self.lSamplesViewText.setText(_translate("SignalFrame", "Samples in view"))
from urh.ui.views.EpicGraphicView import EpicGraphicView
from urh.ui.views.SpectrogramGraphicView import SpectrogramGraphicView
from urh.ui.views.TextEditProtocolView import TextEditProtocolView
from . import urh_rc



import numpy as np
from PyQt5.QtGui import QPainterPath, QPen
from PyQt5.QtWidgets import QGraphicsPathItem

from urh import settings
from urh.cythonext import path_creator
from urh.ui.painting.GridScene import GridScene
from urh.ui.painting.SceneManager import SceneManager


class FFTSceneManager(SceneManager):
    def __init__(self, parent, graphic_view=None):
        self.peak = []

        super().__init__(parent)
        self.scene = GridScene(parent=graphic_view)
        self.scene.setBackgroundBrush(settings.BGCOLOR)

        self.peak_item = self.scene.addPath(QPainterPath(), QPen(settings.PEAK_COLOR, 0))  # type: QGraphicsPathItem

    def show_scene_section(self, x1: float, x2: float, subpath_ranges=None, colors=None):
        start = int(x1) if x1 > 0 else 0
        end = int(x2) if x2 < self.num_samples else self.num_samples
        paths = path_creator.create_path(np.log10(self.plot_data), start, end)
        self.set_path(paths, colors=None)

        try:
            if len(self.peak) > 0:
                peak_path = path_creator.create_path(np.log10(self.peak), start, end)[0]
                self.peak_item.setPath(peak_path)
        except RuntimeWarning:
            pass

    def init_scene(self, draw_grid=True):
        self.scene.draw_grid = draw_grid

        self.peak = self.plot_data if len(self.peak) < self.num_samples else np.maximum(self.peak, self.plot_data)
        self.scene.setSceneRect(0, -5, self.num_samples, 10)

    def clear_path(self):
        for item in self.scene.items():
            if isinstance(item, QGraphicsPathItem) and item != self.peak_item:
                self.scene.removeItem(item)
                item.setParentItem(None)
                del item

    def clear_peak(self):
        self.peak = []
        if self.peak_item:
            self.peak_item.setPath(QPainterPath())

    def eliminate(self):
        super().eliminate()
        self.peak = None
        self.peak_item = None



import xml.etree.ElementTree as ET

from urh.simulator.SimulatorItem import SimulatorItem
from urh.simulator.SimulatorRule import SimulatorRuleCondition
from urh.util.Formatter import Formatter


class SimulatorCounterAction(SimulatorItem):
    def __init__(self):
        super().__init__()
        self.start = 1
        self.step = 1
        self.__value = self.start

    @property
    def value(self):
        return self.__value

    def reset_value(self):
        self.__value = self.start

    def progress_value(self):
        self.__value += self.step

    def validate(self):
        return True

    def set_parent(self, value):
        if value is not None:
            assert value.parent() is None or isinstance(value, SimulatorRuleCondition)

        super().set_parent(value)

    def to_xml(self):
        attrib = {"start": str(self.start), "step": str(self.step)}
        return ET.Element("simulator_counter_action", attrib=attrib)

    @classmethod
    def from_xml(cls, tag):
        result = SimulatorCounterAction()
        result.start = Formatter.str2val(tag.get("start", "1"), int, 1)
        result.step = Formatter.str2val(tag.get("step", "1"), int, 1)
        return result



# -*- coding: utf-8 -*-

#
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SendRecvDialog(object):
    def setupUi(self, SendRecvDialog):
        SendRecvDialog.setObjectName("SendRecvDialog")
        SendRecvDialog.setWindowModality(QtCore.Qt.NonModal)
        SendRecvDialog.resize(1246, 1123)
        SendRecvDialog.setMouseTracking(False)
        self.verticalLayout = QtWidgets.QVBoxLayout(SendRecvDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(SendRecvDialog)
        self.splitter.setStyleSheet("QSplitter::handle:horizontal {\n"
"margin: 4px 0px;\n"
"    background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1, \n"
"stop:0 rgba(255, 255, 255, 0), \n"
"stop:0.5 rgba(100, 100, 100, 100), \n"
"stop:1 rgba(255, 255, 255, 0));\n"
"image: url(:/icons/icons/splitter_handle_vertical.svg);\n"
"}")
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.scrollArea = QtWidgets.QScrollArea(self.splitter)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents_2 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_2.setGeometry(QtCore.QRect(0, 0, 621, 1101))
        self.scrollAreaWidgetContents_2.setObjectName("scrollAreaWidgetContents_2")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_2)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.groupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents_2)
        self.groupBox.setStyleSheet("QGroupBox\n"
"{\n"
" border: none;\n"
"}")
        self.groupBox.setTitle("")
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setContentsMargins(-1, 0, -1, -1)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.progressBarMessage = QtWidgets.QProgressBar(self.groupBox)
        self.progressBarMessage.setProperty("value", 0)
        self.progressBarMessage.setObjectName("progressBarMessage")
        self.gridLayout_2.addWidget(self.progressBarMessage, 19, 0, 1, 1)
        self.labelCurrentMessage = QtWidgets.QLabel(self.groupBox)
        self.labelCurrentMessage.setObjectName("labelCurrentMessage")
        self.gridLayout_2.addWidget(self.labelCurrentMessage, 18, 0, 1, 1)
        self.lReceiveBufferFullText = QtWidgets.QLabel(self.groupBox)
        self.lReceiveBufferFullText.setObjectName("lReceiveBufferFullText")
        self.gridLayout_2.addWidget(self.lReceiveBufferFullText, 7, 0, 1, 1)
        self.progressBarSample = QtWidgets.QProgressBar(self.groupBox)
        self.progressBarSample.setProperty("value", 0)
        self.progressBarSample.setObjectName("progressBarSample")
        self.gridLayout_2.addWidget(self.progressBarSample, 21, 0, 1, 1)
        self.lSamplesSentText = QtWidgets.QLabel(self.groupBox)
        self.lSamplesSentText.setObjectName("lSamplesSentText")
        self.gridLayout_2.addWidget(self.lSamplesSentText, 20, 0, 1, 1)
        self.lTimeText = QtWidgets.QLabel(self.groupBox)
        self.lTimeText.setObjectName("lTimeText")
        self.gridLayout_2.addWidget(self.lTimeText, 12, 0, 1, 1)
        self.lSamplesCapturedText = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSamplesCapturedText.sizePolicy().hasHeightForWidth())
        self.lSamplesCapturedText.setSizePolicy(sizePolicy)
        self.lSamplesCapturedText.setObjectName("lSamplesCapturedText")
        self.gridLayout_2.addWidget(self.lSamplesCapturedText, 5, 0, 1, 1)
        self.lSignalSizeText = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSignalSizeText.sizePolicy().hasHeightForWidth())
        self.lSignalSizeText.setSizePolicy(sizePolicy)
        self.lSignalSizeText.setObjectName("lSignalSizeText")
        self.gridLayout_2.addWidget(self.lSignalSizeText, 9, 0, 1, 1)
        self.lSamplesCaptured = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSamplesCaptured.sizePolicy().hasHeightForWidth())
        self.lSamplesCaptured.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lSamplesCaptured.setFont(font)
        self.lSamplesCaptured.setAlignment(QtCore.Qt.AlignCenter)
        self.lSamplesCaptured.setObjectName("lSamplesCaptured")
        self.gridLayout_2.addWidget(self.lSamplesCaptured, 6, 0, 1, 2)
        self.lTime = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lTime.setFont(font)
        self.lTime.setAlignment(QtCore.Qt.AlignCenter)
        self.lTime.setObjectName("lTime")
        self.gridLayout_2.addWidget(self.lTime, 15, 0, 1, 2)
        self.lSignalSize = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSignalSize.sizePolicy().hasHeightForWidth())
        self.lSignalSize.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lSignalSize.setFont(font)
        self.lSignalSize.setAlignment(QtCore.Qt.AlignCenter)
        self.lSignalSize.setObjectName("lSignalSize")
        self.gridLayout_2.addWidget(self.lSignalSize, 11, 0, 1, 2)
        self.lblRepeatText = QtWidgets.QLabel(self.groupBox)
        self.lblRepeatText.setObjectName("lblRepeatText")
        self.gridLayout_2.addWidget(self.lblRepeatText, 16, 0, 1, 1)
        self.lblCurrentRepeatValue = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lblCurrentRepeatValue.setFont(font)
        self.lblCurrentRepeatValue.setAlignment(QtCore.Qt.AlignCenter)
        self.lblCurrentRepeatValue.setObjectName("lblCurrentRepeatValue")
        self.gridLayout_2.addWidget(self.lblCurrentRepeatValue, 17, 0, 1, 1)
        self.labelReceiveBufferFull = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelReceiveBufferFull.sizePolicy().hasHeightForWidth())
        self.labelReceiveBufferFull.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.labelReceiveBufferFull.setFont(font)
        self.labelReceiveBufferFull.setAlignment(QtCore.Qt.AlignCenter)
        self.labelReceiveBufferFull.setObjectName("labelReceiveBufferFull")
        self.gridLayout_2.addWidget(self.labelReceiveBufferFull, 8, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.btnStart = QtWidgets.QToolButton(self.groupBox)
        self.btnStart.setMinimumSize(QtCore.QSize(64, 64))
        icon = QtGui.QIcon.fromTheme("media-record")
        self.btnStart.setIcon(icon)
        self.btnStart.setIconSize(QtCore.QSize(32, 32))
        self.btnStart.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.btnStart.setObjectName("btnStart")
        self.horizontalLayout.addWidget(self.btnStart)
        self.btnStop = QtWidgets.QToolButton(self.groupBox)
        self.btnStop.setMinimumSize(QtCore.QSize(64, 64))
        icon = QtGui.QIcon.fromTheme("media-playback-stop")
        self.btnStop.setIcon(icon)
        self.btnStop.setIconSize(QtCore.QSize(32, 32))
        self.btnStop.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.btnStop.setObjectName("btnStop")
        self.horizontalLayout.addWidget(self.btnStop)
        self.btnSave = QtWidgets.QToolButton(self.groupBox)
        self.btnSave.setMinimumSize(QtCore.QSize(64, 64))
        icon = QtGui.QIcon.fromTheme("document-save")
        self.btnSave.setIcon(icon)
        self.btnSave.setIconSize(QtCore.QSize(32, 32))
        self.btnSave.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.btnSave.setObjectName("btnSave")
        self.horizontalLayout.addWidget(self.btnSave)
        self.btnClear = QtWidgets.QToolButton(self.groupBox)
        self.btnClear.setMinimumSize(QtCore.QSize(64, 64))
        icon = QtGui.QIcon.fromTheme("view-refresh")
        self.btnClear.setIcon(icon)
        self.btnClear.setIconSize(QtCore.QSize(32, 32))
        self.btnClear.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.btnClear.setObjectName("btnClear")
        self.horizontalLayout.addWidget(self.btnClear)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.gridLayout_2.addLayout(self.horizontalLayout, 1, 0, 1, 2)
        spacerItem2 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridLayout_2.addItem(spacerItem2, 2, 0, 1, 1)
        self.verticalLayout_8.addWidget(self.groupBox)
        self.txtEditErrors = QtWidgets.QTextEdit(self.scrollAreaWidgetContents_2)
        self.txtEditErrors.setReadOnly(True)
        self.txtEditErrors.setObjectName("txtEditErrors")
        self.verticalLayout_8.addWidget(self.txtEditErrors)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents_2)
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.layoutWidget)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.stackedWidget = QtWidgets.QStackedWidget(self.layoutWidget)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page_receive = QtWidgets.QWidget()
        self.page_receive.setObjectName("page_receive")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.page_receive)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.graphicsViewReceive = LiveGraphicView(self.page_receive)
        self.graphicsViewReceive.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.graphicsViewReceive.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.graphicsViewReceive.setObjectName("graphicsViewReceive")
        self.verticalLayout_2.addWidget(self.graphicsViewReceive)
        self.stackedWidget.addWidget(self.page_receive)
        self.page_send = QtWidgets.QWidget()
        self.page_send.setObjectName("page_send")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.page_send)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.graphicsViewSend = EditableGraphicView(self.page_send)
        self.graphicsViewSend.setMouseTracking(True)
        self.graphicsViewSend.setRenderHints(QtGui.QPainter.Antialiasing|QtGui.QPainter.TextAntialiasing)
        self.graphicsViewSend.setTransformationAnchor(QtWidgets.QGraphicsView.NoAnchor)
        self.graphicsViewSend.setResizeAnchor(QtWidgets.QGraphicsView.NoAnchor)
        self.graphicsViewSend.setObjectName("graphicsViewSend")
        self.verticalLayout_3.addWidget(self.graphicsViewSend)
        self.label_7 = QtWidgets.QLabel(self.page_send)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_3.addWidget(self.label_7)
        self.stackedWidget.addWidget(self.page_send)
        self.page_continuous_send = QtWidgets.QWidget()
        self.page_continuous_send.setObjectName("page_continuous_send")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.page_continuous_send)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.graphicsViewContinuousSend = LiveGraphicView(self.page_continuous_send)
        self.graphicsViewContinuousSend.setRenderHints(QtGui.QPainter.Antialiasing|QtGui.QPainter.TextAntialiasing)
        self.graphicsViewContinuousSend.setObjectName("graphicsViewContinuousSend")
        self.verticalLayout_6.addWidget(self.graphicsViewContinuousSend)
        self.stackedWidget.addWidget(self.page_continuous_send)
        self.page_spectrum = QtWidgets.QWidget()
        self.page_spectrum.setObjectName("page_spectrum")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.page_spectrum)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.graphicsViewFFT = LiveGraphicView(self.page_spectrum)
        self.graphicsViewFFT.setObjectName("graphicsViewFFT")
        self.verticalLayout_7.addWidget(self.graphicsViewFFT)
        self.graphicsViewSpectrogram = QtWidgets.QGraphicsView(self.page_spectrum)
        self.graphicsViewSpectrogram.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.graphicsViewSpectrogram.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.graphicsViewSpectrogram.setRenderHints(QtGui.QPainter.SmoothPixmapTransform|QtGui.QPainter.TextAntialiasing)
        self.graphicsViewSpectrogram.setCacheMode(QtWidgets.QGraphicsView.CacheNone)
        self.graphicsViewSpectrogram.setViewportUpdateMode(QtWidgets.QGraphicsView.MinimalViewportUpdate)
        self.graphicsViewSpectrogram.setObjectName("graphicsViewSpectrogram")
        self.verticalLayout_7.addWidget(self.graphicsViewSpectrogram)
        self.verticalLayout_7.setStretch(0, 1)
        self.verticalLayout_7.setStretch(1, 1)
        self.stackedWidget.addWidget(self.page_spectrum)
        self.page_sniff = QtWidgets.QWidget()
        self.page_sniff.setObjectName("page_sniff")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.page_sniff)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.graphicsView_sniff_Preview = LiveGraphicView(self.page_sniff)
        self.graphicsView_sniff_Preview.setObjectName("graphicsView_sniff_Preview")
        self.verticalLayout_4.addWidget(self.graphicsView_sniff_Preview)
        self.txtEd_sniff_Preview = QtWidgets.QPlainTextEdit(self.page_sniff)
        self.txtEd_sniff_Preview.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.txtEd_sniff_Preview.setReadOnly(True)
        self.txtEd_sniff_Preview.setMaximumBlockCount(100)
        self.txtEd_sniff_Preview.setObjectName("txtEd_sniff_Preview")
        self.verticalLayout_4.addWidget(self.txtEd_sniff_Preview)
        self.btnAccept = QtWidgets.QPushButton(self.page_sniff)
        self.btnAccept.setAutoDefault(False)
        self.btnAccept.setObjectName("btnAccept")
        self.verticalLayout_4.addWidget(self.btnAccept)
        self.stackedWidget.addWidget(self.page_sniff)
        self.horizontalLayout_2.addWidget(self.stackedWidget)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_y_scale = QtWidgets.QLabel(self.layoutWidget)
        self.label_y_scale.setObjectName("label_y_scale")
        self.verticalLayout_5.addWidget(self.label_y_scale)
        self.sliderYscale = QtWidgets.QSlider(self.layoutWidget)
        self.sliderYscale.setMinimum(1)
        self.sliderYscale.setMaximum(1000)
        self.sliderYscale.setProperty("value", 1)
        self.sliderYscale.setOrientation(QtCore.Qt.Vertical)
        self.sliderYscale.setTickInterval(1)
        self.sliderYscale.setObjectName("sliderYscale")
        self.verticalLayout_5.addWidget(self.sliderYscale)
        self.horizontalLayout_2.addLayout(self.verticalLayout_5)
        self.verticalLayout.addWidget(self.splitter)

        self.retranslateUi(SendRecvDialog)
        self.stackedWidget.setCurrentIndex(4)
        SendRecvDialog.setTabOrder(self.btnStart, self.btnStop)
        SendRecvDialog.setTabOrder(self.btnStop, self.btnSave)
        SendRecvDialog.setTabOrder(self.btnSave, self.btnClear)
        SendRecvDialog.setTabOrder(self.btnClear, self.txtEd_sniff_Preview)
        SendRecvDialog.setTabOrder(self.txtEd_sniff_Preview, self.sliderYscale)
        SendRecvDialog.setTabOrder(self.sliderYscale, self.txtEditErrors)
        SendRecvDialog.setTabOrder(self.txtEditErrors, self.graphicsViewSend)
        SendRecvDialog.setTabOrder(self.graphicsViewSend, self.graphicsViewReceive)
        SendRecvDialog.setTabOrder(self.graphicsViewReceive, self.btnAccept)

    def retranslateUi(self, SendRecvDialog):
        _translate = QtCore.QCoreApplication.translate
        SendRecvDialog.setWindowTitle(_translate("SendRecvDialog", "Record Signal"))
        self.progressBarMessage.setFormat(_translate("SendRecvDialog", "%v/%m"))
        self.labelCurrentMessage.setText(_translate("SendRecvDialog", "Current message:"))
        self.lReceiveBufferFullText.setText(_translate("SendRecvDialog", "Receive buffer full:"))
        self.progressBarSample.setFormat(_translate("SendRecvDialog", "%v/%m"))
        self.lSamplesSentText.setText(_translate("SendRecvDialog", "Current sample:"))
        self.lTimeText.setText(_translate("SendRecvDialog", "Time (in seconds):"))
        self.lSamplesCapturedText.setText(_translate("SendRecvDialog", "Samples captured:"))
        self.lSignalSizeText.setText(_translate("SendRecvDialog", "Signal size (in MiB):"))
        self.lSamplesCaptured.setText(_translate("SendRecvDialog", "0"))
        self.lTime.setText(_translate("SendRecvDialog", "0"))
        self.lSignalSize.setText(_translate("SendRecvDialog", "0"))
        self.lblRepeatText.setText(_translate("SendRecvDialog", "Current iteration:"))
        self.lblCurrentRepeatValue.setText(_translate("SendRecvDialog", "0"))
        self.labelReceiveBufferFull.setText(_translate("SendRecvDialog", "0%"))
        self.btnStart.setToolTip(_translate("SendRecvDialog", "Record signal"))
        self.btnStart.setText(_translate("SendRecvDialog", "Start"))
        self.btnStop.setToolTip(_translate("SendRecvDialog", "Stop recording"))
        self.btnStop.setText(_translate("SendRecvDialog", "Stop"))
        self.btnSave.setText(_translate("SendRecvDialog", "Save..."))
        self.btnClear.setToolTip(_translate("SendRecvDialog", "Clear"))
        self.btnClear.setText(_translate("SendRecvDialog", "Clear"))
        self.label_7.setText(_translate("SendRecvDialog", "Hint: You can edit the raw signal before sending."))
        self.btnAccept.setToolTip(_translate("SendRecvDialog", "<html><head/><body><p>Accept the sniffed data and load it into <span style=\" font-weight:600;\">Analysis</span> tab.</p></body></html>"))
        self.btnAccept.setText(_translate("SendRecvDialog", "Accept data (Open in Analysis)"))
        self.label_y_scale.setText(_translate("SendRecvDialog", "Y-Scale"))
from urh.ui.views.EditableGraphicView import EditableGraphicView
from urh.ui.views.LiveGraphicView import LiveGraphicView
from . import urh_rc



from urh.ui.KillerDoubleSpinBox import KillerDoubleSpinBox
from urh.ui.delegates.SpinBoxDelegate import SpinBoxDelegate


class KillerSpinBoxDelegate(SpinBoxDelegate):
    def __init__(self, minimum, maximum, parent=None, suffix=""):
        super().__init__(minimum, maximum, parent, suffix)

    def _get_editor(self, parent):
        editor = KillerDoubleSpinBox(parent)
        editor.setDecimals(3)
        return editor



# -*- coding: utf-8 -*-

#
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogAdvancedModSettings(object):
    def setupUi(self, DialogAdvancedModSettings):
        DialogAdvancedModSettings.setObjectName("DialogAdvancedModSettings")
        DialogAdvancedModSettings.resize(527, 501)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(DialogAdvancedModSettings)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(DialogAdvancedModSettings)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.spinBoxPauseThreshold = QtWidgets.QSpinBox(self.groupBox)
        self.spinBoxPauseThreshold.setMinimum(0)
        self.spinBoxPauseThreshold.setMaximum(999999999)
        self.spinBoxPauseThreshold.setProperty("value", 8)
        self.spinBoxPauseThreshold.setObjectName("spinBoxPauseThreshold")
        self.verticalLayout.addWidget(self.spinBoxPauseThreshold)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.groupBox_2 = QtWidgets.QGroupBox(DialogAdvancedModSettings)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.groupBox_2)
        self.label_2.setWordWrap(True)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.spinBoxMessageLengthDivisor = QtWidgets.QSpinBox(self.groupBox_2)
        self.spinBoxMessageLengthDivisor.setMinimum(1)
        self.spinBoxMessageLengthDivisor.setMaximum(999999999)
        self.spinBoxMessageLengthDivisor.setObjectName("spinBoxMessageLengthDivisor")
        self.verticalLayout_2.addWidget(self.spinBoxMessageLengthDivisor)
        self.verticalLayout_3.addWidget(self.groupBox_2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(DialogAdvancedModSettings)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_3.addWidget(self.buttonBox)

        self.retranslateUi(DialogAdvancedModSettings)
        self.buttonBox.accepted.connect(DialogAdvancedModSettings.accept)
        self.buttonBox.rejected.connect(DialogAdvancedModSettings.reject)

    def retranslateUi(self, DialogAdvancedModSettings):
        _translate = QtCore.QCoreApplication.translate
        DialogAdvancedModSettings.setWindowTitle(_translate("DialogAdvancedModSettings", "Advanced Modulation Settings"))
        self.groupBox.setTitle(_translate("DialogAdvancedModSettings", "Pause Threshold"))
        self.label.setText(_translate("DialogAdvancedModSettings", "<html><head/><body><p>The pause threshold gives you control <span style=\" font-weight:600;\">when to insert a message break.</span></p><p>The pause threshold is the maximum length of consecutive zero bits represented by a pause before a new message begins.</p><p>Special value is 0 to disable message breaking completely.</p></body></html>"))
        self.spinBoxPauseThreshold.setSpecialValueText(_translate("DialogAdvancedModSettings", "Disable"))
        self.groupBox_2.setTitle(_translate("DialogAdvancedModSettings", "Message Length Divisor"))
        self.label_2.setText(_translate("DialogAdvancedModSettings", "<html><head/><body><p>With the message <span style=\" font-weight:600;\">divisor length</span> you can control the minimum message length in a flexible way. URH will try to demodulate signals in such a way, that the resulting message has a number of bits that is divisble by the configured divisor. <br/><br/><span style=\" font-style:italic;\">How does the zero padding work? Remaining zero bits are taken from the pause behind the message if possible.</span></p></body></html>"))



# noinspection PyUnresolvedReferences
cimport numpy as np
import numpy as np
from cpython cimport array
import array
import cython

from cython.parallel import prange
from libc.stdlib cimport malloc, free
from libcpp.algorithm cimport sort
from libc.stdint cimport uint64_t

cpdef tuple k_means(float[:] data, unsigned int k=2):
    cdef float[:] centers = np.empty(k, dtype=np.float32)
    cdef list clusters = []
    cdef set unique = set(data)
    cdef unsigned long i

    if len(unique) < k:
        print("Warning: less different values than k")
        k = len(unique)

    for i in range(k):
        centers[i] = unique.pop()
        clusters.append([])

    cdef float[:] old_centers = np.array(centers, dtype=np.float32)
    cdef float distance, min_distance, error = 1.0
    cdef unsigned int j, index = 0, N = len(data)

    while error != 0:
        for i in range(k):
            clusters[i].clear()

        for i in range(N):
            min_distance = 999999999
            for j in range(k):
                distance = (centers[j] - data[i]) * (centers[j] - data[i])
                if distance < min_distance:
                    min_distance = distance
                    index = j
            clusters[index].append(data[i])

        old_centers = np.array(centers)
        for i in range(k):
            centers[i] = np.mean(clusters[i])

        error = 0.0
        for i in range(k):
            error += old_centers[i] * old_centers[i] - centers[i] * centers[i]

    return centers, clusters


def segment_messages_from_magnitudes(cython.floating[:] magnitudes, float noise_threshold):
    """
    Get the list of start, end indices of messages

    :param magnitudes: Magnitudes of samples
    :param q: Factor which controls how many samples of previous above noise plateau must be under noise to be counted as noise
    :return:
    """
    cdef list result = []

    if len(magnitudes) == 0:
        return []

    cdef unsigned long i, N = len(magnitudes), start = 0
    cdef unsigned long summed_message_samples = 0

    # tolerance / robustness against outliers
    cdef unsigned int outlier_tolerance = 10
    cdef unsigned int conseq_above = 0, conseq_below = 0

    # Three states: 1 = above noise, 0 = in noise, but not yet above k threshold (k * above_total), -1 = in noise
    cdef int state
    state = 1 if magnitudes[0] > noise_threshold else -1

    cdef bint is_above_noise

    for i in range(N):
        is_above_noise = magnitudes[i] > noise_threshold
        if state == 1:
            if is_above_noise:
                conseq_below = 0
            else:
                conseq_below += 1
        elif state == -1:
            if is_above_noise:
                conseq_above += 1
            else:
                conseq_above = 0

        # Perform state change if necessary
        if state == 1 and conseq_below >= outlier_tolerance:
            # 1 -> -1
            state = -1
            result.append((start, i - conseq_below))
            summed_message_samples += (i-conseq_below) - start
            conseq_below = conseq_above = 0
        elif state == -1 and conseq_above >= outlier_tolerance:
            # -1 -> 1
            state = 1
            start = i - conseq_above
            conseq_below = conseq_above = 0

    # append last message
    if state == 1 and start < N - conseq_below:
        result.append((start, N - conseq_below))

    return result

cpdef uint64_t[:] get_threshold_divisor_histogram(uint64_t[:] plateau_lengths, float threshold=0.2):
    """
    Get a histogram (i.e. count) how many times a value is a threshold divisor for other values in given data
    
    Threshold divisible is defined as having a decimal place less than .2 (threshold)
    
    :param plateau_lengths: 
    :return: 
    """
    cdef uint64_t i, j, x, y, minimum, maximum, num_lengths = len(plateau_lengths)

    cdef np.ndarray[np.uint64_t, ndim=1] histogram = np.zeros(int(np.max(plateau_lengths)) + 1, dtype=np.uint64)

    for i in range(0, num_lengths):
        for j in range(i+1, num_lengths):
            x = plateau_lengths[i]
            y = plateau_lengths[j]
            if x == 0 or y == 0:
                continue

            if x < y:
                minimum = x
                maximum = y
            else:
                minimum = y
                maximum = x

            if maximum / <double>minimum - (maximum / minimum) < threshold:
                histogram[minimum] += 1

    return histogram

cpdef np.ndarray[np.uint64_t, ndim=1] merge_plateaus(np.ndarray[np.uint64_t, ndim=1] plateaus,
                                                     uint64_t tolerance,
                                                     uint64_t max_count):
    cdef uint64_t j, n, L = len(plateaus), current = 0, i = 1, tmp_sum
    if L == 0:
        return np.zeros(0, dtype=np.uint64)

    cdef np.ndarray[np.uint64_t, ndim=1] result = np.empty(L, dtype=np.uint64)
    if plateaus[0] <= tolerance:
        result[0] = 0
    else:
        result[0] = plateaus[0]

    while i < L and current < max_count:
        if plateaus[i] <= tolerance:
            # Look ahead to see whether we need to merge a larger window e.g. for 67, 1, 10, 1, 21
            n = 2
            while i + n < L and plateaus[i + n] <= tolerance:
                n += 2

            tmp_sum = 0
            for j in range(i - 1, min(L, i + n)):
                tmp_sum += plateaus[j]

            result[current] = tmp_sum
            i += n
        else:
            current += 1
            result[current] = plateaus[i]
            i += 1

    return result[:current+1]


cpdef np.ndarray[np.uint64_t, ndim=1] get_plateau_lengths(float[:] rect_data, float center, int percentage=25):
    if len(rect_data) == 0 or center is None:
        return np.array([], dtype=np.uint64)

    cdef int state, new_state
    state = -1 if rect_data[0] <= center else 1
    cdef unsigned long long plateau_length = 0
    cdef unsigned long long current_sum = 0
    cdef unsigned long long i = 0
    cdef unsigned long long len_data = len(rect_data)
    cdef float sample

    cdef array.array result = array.array('Q', [])

    for i in range(0, len_data):
        if current_sum >= percentage * len_data / 100:
            break

        sample = rect_data[i]
        new_state = -1 if sample <= center else 1

        if state == new_state:
            plateau_length += 1
        else:
            result.append(plateau_length)
            current_sum += plateau_length
            state = new_state
            plateau_length = 1

    return np.array(result, dtype=np.uint64)


cdef float median(double[:] data, unsigned long start, unsigned long data_len, unsigned int k=3) nogil:
    cdef unsigned long i, j

    if start + k > data_len:
        k = data_len - start

    cdef float* buffer = <float *>malloc(k * sizeof(float))
    for i in range(0, k):
        buffer[i] = data[start+i]

    sort(&buffer[0], (&buffer[0]) + k)
    try:
        return buffer[k//2]
    finally:
        free(buffer)

cpdef np.ndarray[np.float32_t, ndim=1] median_filter(double[:] data, unsigned int k=3):
    cdef long long start, end, i, n = len(data)

    cdef np.ndarray[np.float32_t, ndim=1] result = np.zeros(n, dtype=np.float32)

    for i in prange(0, n, nogil=True, schedule='static'):
        if i < k // 2:
            start = 0
        else:
            start = i - k // 2

        result[i] = median(data, start=i, data_len=n, k=k)

    return result



from urh.signalprocessing.ChecksumLabel import ChecksumLabel

from urh.signalprocessing.FieldType import FieldType
from urh.signalprocessing.MessageType import MessageType
from urh.signalprocessing.ProtocoLabel import ProtocolLabel


class MessageTypeBuilder(object):
    def __init__(self, name: str):
        self.name = name
        self.message_type = MessageType(name)

    def add_label(self, label_type: FieldType.Function, length: int, name: str=None):
        try:
            start = self.message_type[-1].end
            color_index = self.message_type[-1].color_index + 1
        except IndexError:
            start, color_index = 0, 0

        if name is None:
            name = label_type.value

        lbl = ProtocolLabel(name, start, start+length-1, color_index, field_type=FieldType(label_type.name, label_type))
        self.message_type.append(lbl)

    def add_checksum_label(self, length, checksum, data_start=None, data_end=None, name: str=None):
        label_type = FieldType.Function.CHECKSUM
        try:
            start = self.message_type[-1].end
            color_index = self.message_type[-1].color_index + 1
        except IndexError:
            start, color_index = 0, 0

        if name is None:
            name = label_type.value

        if data_start is None:
            # End of sync or preamble
            sync_label = self.message_type.get_first_label_with_type(FieldType.Function.SYNC)
            if sync_label:
                data_start = sync_label.end
            else:
                preamble_label = self.message_type.get_first_label_with_type(FieldType.Function.PREAMBLE)
                if preamble_label:
                    data_start = preamble_label.end
                else:
                    data_start = 0

        if data_end is None:
            data_end = start

        lbl = ChecksumLabel(name, start, start+length-1, color_index, field_type=FieldType(label_type.name, label_type))
        lbl.data_ranges = [(data_start, data_end)]
        lbl.checksum = checksum
        self.message_type.append(lbl)



# noinspection PyUnresolvedReferences
cimport numpy as np
import numpy as np


from libc.math cimport floor, ceil, pow
from libc.stdlib cimport malloc, free

from libcpp cimport bool
from libc.stdint cimport uint8_t, uint32_t, int32_t, int64_t

from urh.cythonext.util import crc
from urh.cythonext.util cimport bit_array_to_number

cpdef set find_longest_common_sub_sequence_indices(np.uint8_t[::1] seq1, np.uint8_t[::1] seq2):
    cdef unsigned int i, j, longest = 0, counter = 0, len_bits1 = len(seq1), len_bits2 = len(seq2)
    cdef unsigned short max_results = 10, current_result = 0

    cdef unsigned int[:, ::1] m = np.zeros((len_bits1+1, len_bits2+1), dtype=np.uint32, order="C")
    cdef unsigned int[:, ::1] result_indices = np.zeros((max_results, 2), dtype=np.uint32, order="C")

    for i in range(0, len_bits1):
        for j in range(0, len_bits2):
            if seq1[i] == seq2[j]:
                counter = m[i, j] + 1
                m[i+1, j+1] = counter

                if counter > longest:
                    longest = counter

                    current_result = 0
                    result_indices[current_result, 0] = i - counter + 1
                    result_indices[current_result, 1] = i + 1
                elif counter == longest:
                    if current_result < max_results - 1:
                        current_result += 1
                        result_indices[current_result, 0] = i - counter + 1
                        result_indices[current_result, 1] = i + 1

    cdef set result = set()
    for i in range(current_result+1):
        result.add((result_indices[i, 0], result_indices[i, 1]))

    return result

cpdef uint32_t find_first_difference(uint8_t[::1] bits1, uint8_t[::1] bits2, uint32_t len_bits1, uint32_t len_bits2) nogil:
    cdef uint32_t i, smaller_len = min(len_bits1, len_bits2)

    for i in range(0, smaller_len):
        if bits1[i] != bits2[i]:
            return i

    return smaller_len

cpdef np.ndarray[np.uint32_t, ndim=2, mode="c"] get_difference_matrix(list bitvectors):
    cdef uint32_t i, j, N = len(bitvectors)
    cdef np.ndarray[np.uint32_t, ndim=2, mode="c"] result = np.zeros((N, N), dtype=np.uint32, order="C")

    cdef uint8_t[::1] bitvector_i
    cdef uint32_t len_bitvector_i

    for i in range(N):
        bitvector_i = bitvectors[i]
        len_bitvector_i = len(bitvector_i)
        for j in range(i + 1, N):
            result[i, j] = find_first_difference(bitvector_i, bitvectors[j], len_bitvector_i, len(bitvectors[j]))

    return result

cpdef list get_hexvectors(list bitvectors):
    cdef list result = []
    cdef uint8_t[::1] bitvector
    cdef size_t i, j, M, N = len(bitvectors)

    cdef np.ndarray[np.uint8_t, mode="c"] hexvector
    cdef size_t len_bitvector

    for i in range(0, N):
        bitvector = bitvectors[i]
        len_bitvector = len(bitvector)

        M = <size_t>ceil(len_bitvector / 4)
        hexvector = np.zeros(M, dtype=np.uint8, order="C")

        for j in range(0, M):
            hexvector[j] = bit_array_to_number(bitvector, min(len_bitvector, 4*j+4), 4*j)

        result.append(hexvector)

    return result


cdef int lower_multiple_of_n(int number, int n) nogil:
        return n * <int>floor(number / n)

cdef int64_t find(uint8_t[:] data, int64_t len_data, uint8_t element, int64_t start=0) nogil:
    cdef int64_t i
    for i in range(start, len_data):
        if data[i] == element:
            return i
    return -1

cpdef tuple get_raw_preamble_position(uint8_t[:] bitvector):
    cdef int64_t N = len(bitvector)
    if N == 0:
        return 0, 0

    cdef int64_t i, j, n, m, start = -1
    cdef double k = 0

    cdef int64_t lower = 0, upper = 0
    cdef uint8_t a, b

    cdef uint8_t* preamble_pattern = NULL
    cdef int64_t len_preamble_pattern, preamble_end

    cdef bool preamble_end_reached

    while k < 2 and start < N:
        start += 1

        a = bitvector[start]
        b = 1 if a == 0 else 0

        # now we search for the pattern a^n b^m
        n = find(bitvector, N, b, start) - start

        if n <= 0:
            return 0, 0, 0

        m = find(bitvector, N, a, start+n) - n - start

        if m <= 0:
            return 0, 0, 0

        #preamble_pattern = a * n + b * m
        len_preamble_pattern = n + m
        preamble_pattern = <uint8_t*> malloc(len_preamble_pattern * sizeof(uint8_t))

        for j in range(0, n):
            preamble_pattern[j] = a
        for j in range(n, len_preamble_pattern):
            preamble_pattern[j] = b

        preamble_end = start
        preamble_end_reached = False
        for i in range(start, N, len_preamble_pattern):
            if preamble_end_reached:
                break
            for j in range(0, len_preamble_pattern):
                if bitvector[i+j] != preamble_pattern[j]:
                    preamble_end_reached = True
                    preamble_end = i
                    break

        free(preamble_pattern)

        upper = start + lower_multiple_of_n(preamble_end + 1 - start, len_preamble_pattern)
        lower = upper - len_preamble_pattern

        k = (upper - start) / len_preamble_pattern

    if k > 2:
        return start, lower, upper
    else:
        # no preamble found
        return 0, 0, 0


cpdef dict find_possible_sync_words(np.ndarray[np.uint32_t, ndim=2, mode="c"] difference_matrix,
                                    np.ndarray[np.uint32_t, ndim=2, mode="c"] raw_preamble_positions,
                                    list bitvectors, int n_gram_length):
    cdef dict possible_sync_words = dict()

    cdef uint32_t i, j, num_rows = difference_matrix.shape[0], num_cols = difference_matrix.shape[1]
    cdef uint32_t sync_len, sync_end, start, index, k, n

    cdef bytes sync_word

    cdef np.ndarray[np.uint8_t, mode="c"] bitvector

    cdef uint8_t ij_ctr = 0
    cdef uint32_t* ij_arr = <uint32_t*>malloc(2 * sizeof(uint32_t))
    cdef uint8_t* temp = NULL

    for i in range(0, num_rows):
        for j in range(i + 1, num_cols):
            # position of first difference between message i and j
            sync_end = difference_matrix[i, j]

            if sync_end == 0:
                continue

            ij_arr[0] = i
            ij_arr[1] = j

            for k in range(0, 2):
                for ij_ctr in range(0, 2):
                    index = ij_arr[ij_ctr]
                    start = raw_preamble_positions[index, 0] + raw_preamble_positions[index, k + 1]

                    # We take the next lower multiple of n for the sync len
                    # In doubt, it is better to under estimate the sync len to prevent it from
                    # taking needed values from other fields e.g. leading zeros for a length field
                    sync_len = max(0, lower_multiple_of_n(sync_end - start, n_gram_length))

                    if sync_len >= 2:
                        bitvector = bitvectors[index]
                        if sync_len == 2:
                            # Sync word must not be empty or just two bits long and "10" or "01" because
                            # that would be indistinguishable from the preamble
                            if bitvector[start] == 0 and bitvector[start+1] == 1:
                                continue
                            if bitvector[start] == 1 and bitvector[start+1] == 0:
                                continue

                        temp = <uint8_t*>malloc(sync_len * sizeof(uint8_t))
                        for n in range(0, sync_len):
                            temp[n] = bitvector[start+n]
                        sync_word = <bytes> temp[:sync_len]
                        free(temp)

                        possible_sync_words.setdefault(sync_word, 0)
                        if (start + sync_len) % n_gram_length == 0:
                            # if sync end aligns nicely at n gram length give it a larger score
                            possible_sync_words[sync_word] += 1
                        else:
                            possible_sync_words[sync_word] += 0.5

    free(ij_arr)
    return possible_sync_words

cpdef np.ndarray[np.float64_t] create_difference_histogram(list vectors, list active_indices):
    """
    Return a histogram of common ranges. E.g. [1, 1, 0.75, 0.8] means 75% of values at third column are equal
    
    :param vectors: Vectors over which differences the histogram will be created
    :param active_indices: Active indices of vectors. Vectors with index not in this list will be ignored
    :return: 
    """
    cdef unsigned long i,j,k,index_i,index_j, L = len(active_indices)
    cdef unsigned long longest = 0, len_vector, len_vector_i
    for i in active_indices:
        len_vector = len(vectors[i])
        if len_vector > longest:
            longest = len_vector

    cdef np.ndarray[np.float64_t] histogram = np.zeros(longest, dtype=np.float64)
    cdef double n = (len(active_indices) * (len(active_indices) - 1)) // 2

    cdef np.ndarray[np.uint8_t] bitvector_i, bitvector_j

    for i in range(0, L - 1):
        index_i = active_indices[i]
        bitvector_i = vectors[index_i]
        len_vector_i = len(bitvector_i)
        for j in range(i+1, L):
            index_j = active_indices[j]
            bitvector_j = vectors[index_j]
            for k in range(0, <size_t>min(len_vector_i, <size_t>len(bitvector_j))):
                if bitvector_i[k] == bitvector_j[k]:
                    histogram[k] += 1 / n
    return histogram

cpdef list find_occurrences(np.uint8_t[::1] a, np.uint8_t[::1] b,
                            unsigned long[:] ignore_indices=None, bool return_after_first=False):
    """
    Find the indices of occurrences of b in a. 
    
    :param a: Larger array
    :param b: Subarray to search for
    :return: List of start indices of b in a 
    """
    cdef unsigned long i, j
    cdef unsigned long len_a = len(a), len_b = len(b)

    cdef bool ignore_indices_present = ignore_indices is not None

    if len_b > len_a:
        return []

    cdef list result = []
    cdef bool found
    for i in range(0, (len_a-len_b) + 1):
        found = True
        for j in range(0, len_b):
            if ignore_indices_present:
                if i+j in ignore_indices:
                    found = False
                    break

            if a[i+j] != b[j]:
                found = False
                break
        if found:
            if return_after_first:
                return [i]
            else:
                result.append(i)

    return result

cpdef np.ndarray[np.int32_t, ndim=2, mode="c"] create_seq_number_difference_matrix(list bitvectors, int n_gram_length):
    """
    Create the difference matrix e.g.
    10 20 0
    1  2  3
    4  5  6

    means first eight bits of messages 1 and 2 (row 1) differ by 10 if they are considered as decimal number

    :type bitvectors: list of np.ndarray
    :type n_gram_length: int
    :rtype: np.ndarray
    """
    cdef size_t max_len = len(max(bitvectors, key=len))
    cdef size_t i, j, k, index, N = len(bitvectors), M = <size_t>ceil(max_len / n_gram_length)
    cdef uint8_t[::1] bv1, bv2
    cdef size_t len_bv1, len_bv2
    cdef int32_t diff
    cdef int32_t n_gram_power_two = <int32_t>pow(2, n_gram_length)

    cdef np.ndarray[np.int32_t, ndim=2, mode="c"] result = np.full((N - 1, M), -1, dtype=np.int32)
    for i in range(1, N):
        bv1 = bitvectors[i - 1]
        bv2 = bitvectors[i]
        len_bv1 = len(bv1)
        len_bv2 = len(bv2)
        k = min(len_bv1, len_bv2)
        for j in range(0, k, n_gram_length):
            index = j / n_gram_length
            if index < M:
                diff = bit_array_to_number(bv2, min(len_bv2, j + n_gram_length), j) -\
                       bit_array_to_number(bv1, min(len_bv1, j+n_gram_length), j)
                # add + n_gram_power_two because in C modulo can be negative
                result[i - 1, index] = (diff + n_gram_power_two) % n_gram_power_two

    return result

cpdef set check_crc_for_messages(list message_indices, list bitvectors,
                                 unsigned long data_start, unsigned long data_stop,
                                 unsigned long crc_start, unsigned long crc_stop,
                                 unsigned char[:] crc_polynomial, unsigned char[:] crc_start_value,
                                 unsigned char[:] crc_final_xor,
                                 bool crc_lsb_first, bool crc_reverse_polynomial,
                                 bool crc_reverse_all, bool crc_little_endian):
    """
    Check a configurable subset of bitvectors for a matching CRC and return the indices of the 
    vectors who match the CRC with the given parameters
    :return: 
    """
    cdef set result = set()
    cdef unsigned long j, index, end = len(message_indices)
    cdef np.ndarray[np.uint8_t] bits
    cdef unsigned char[:] crc_input
    cdef unsigned long long check

    for j in range(0, end):
        index = message_indices[j]
        bits = bitvectors[index]
        crc_input = bits[data_start:data_stop]
        #check = int("".join(map(str, bits[crc_start:crc_stop])), 2)
        check = bit_array_to_number(bits[crc_start:crc_stop], crc_stop - crc_start)
        if crc(crc_input, crc_polynomial, crc_start_value, crc_final_xor,
               crc_lsb_first, crc_reverse_polynomial,
               crc_reverse_all, crc_little_endian) == check:
            result.add(index)

    return result



from PyQt5.QtGui import QValidator
from PyQt5.QtCore import pyqtSignal

class RuleExpressionValidator(QValidator):
    validation_status_changed = pyqtSignal(QValidator.State, str)

    def __init__(self, sim_expression_parser, is_formula=True, parent=None):
        super().__init__(parent)
        self.parser = sim_expression_parser
        self.is_formula = is_formula

    def validate(self, text, pos):
        valid, message, _ = self.parser.validate_expression(text, self.is_formula)
        state = QValidator.Acceptable if valid else QValidator.Intermediate

        self.validation_status_changed.emit(state, message)
        return (state, text, pos)


# -*- coding: utf-8 -*-

#
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogMessageType(object):
    def setupUi(self, DialogMessageType):
        DialogMessageType.setObjectName("DialogMessageType")
        DialogMessageType.resize(471, 359)
        self.gridLayout = QtWidgets.QGridLayout(DialogMessageType)
        self.gridLayout.setObjectName("gridLayout")
        self.cbRulesetMode = QtWidgets.QComboBox(DialogMessageType)
        self.cbRulesetMode.setObjectName("cbRulesetMode")
        self.cbRulesetMode.addItem("")
        self.cbRulesetMode.addItem("")
        self.cbRulesetMode.addItem("")
        self.gridLayout.addWidget(self.cbRulesetMode, 1, 0, 1, 2)
        self.tblViewRuleset = QtWidgets.QTableView(DialogMessageType)
        self.tblViewRuleset.setShowGrid(False)
        self.tblViewRuleset.setObjectName("tblViewRuleset")
        self.gridLayout.addWidget(self.tblViewRuleset, 2, 0, 3, 2)
        self.btnRemoveRule = QtWidgets.QToolButton(DialogMessageType)
        icon = QtGui.QIcon.fromTheme("list-remove")
        self.btnRemoveRule.setIcon(icon)
        self.btnRemoveRule.setObjectName("btnRemoveRule")
        self.gridLayout.addWidget(self.btnRemoveRule, 3, 2, 1, 1)
        self.rbAssignManually = QtWidgets.QRadioButton(DialogMessageType)
        self.rbAssignManually.setObjectName("rbAssignManually")
        self.gridLayout.addWidget(self.rbAssignManually, 0, 0, 1, 1)
        self.rbAssignAutomatically = QtWidgets.QRadioButton(DialogMessageType)
        self.rbAssignAutomatically.setObjectName("rbAssignAutomatically")
        self.gridLayout.addWidget(self.rbAssignAutomatically, 0, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 145, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 4, 2, 1, 1)
        self.btnAddRule = QtWidgets.QToolButton(DialogMessageType)
        icon = QtGui.QIcon.fromTheme("list-add")
        self.btnAddRule.setIcon(icon)
        self.btnAddRule.setObjectName("btnAddRule")
        self.gridLayout.addWidget(self.btnAddRule, 2, 2, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(DialogMessageType)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 5, 0, 1, 2)

        self.retranslateUi(DialogMessageType)

    def retranslateUi(self, DialogMessageType):
        _translate = QtCore.QCoreApplication.translate
        DialogMessageType.setWindowTitle(_translate("DialogMessageType", "Dialog"))
        self.cbRulesetMode.setItemText(0, _translate("DialogMessageType", "All rules must apply (AND)"))
        self.cbRulesetMode.setItemText(1, _translate("DialogMessageType", "At least one rule must apply (OR)"))
        self.cbRulesetMode.setItemText(2, _translate("DialogMessageType", "No rule must apply (NOR)"))
        self.btnRemoveRule.setToolTip(_translate("DialogMessageType", "Remove ruleset"))
        self.btnRemoveRule.setText(_translate("DialogMessageType", "..."))
        self.rbAssignManually.setText(_translate("DialogMessageType", "Assi&gn manually"))
        self.rbAssignAutomatically.setText(_translate("DialogMessageType", "Assign a&utomatically"))
        self.btnAddRule.setToolTip(_translate("DialogMessageType", "Add ruleset"))
        self.btnAddRule.setText(_translate("DialogMessageType", "..."))



import numpy as np

from urh.cythonext import util
from urh.signalprocessing.Message import Message


def auto_assign_participants(messages, participants):
    """

    :type messages: list of Message
    :type participants: list of Participant
    :return:
    """
    if len(participants) == 0:
        return

    if len(participants) == 1:
        for message in messages:  # type: Message
            message.participant = participants[0]
        return

    # Try to assign participants based on SRC_ADDRESS label and participant address
    for msg in filter(lambda m: m.participant is None, messages):
        src_address = msg.get_src_address_from_data()
        if src_address:
            try:
                msg.participant = next(p for p in participants if p.address_hex == src_address)
            except StopIteration:
                pass

    # Assign remaining participants based on RSSI of messages
    rssis = np.array([msg.rssi for msg in messages], dtype=np.float32)
    min_rssi, max_rssi = util.minmax(rssis)
    center_spacing = (max_rssi - min_rssi) / (len(participants) - 1)
    centers = [min_rssi + i * center_spacing for i in range(0, len(participants))]
    rssi_assigned_centers = []

    for rssi in rssis:
        center_index = np.argmin(np.abs(rssi - centers))
        rssi_assigned_centers.append(int(center_index))

    participants.sort(key=lambda participant: participant.relative_rssi)
    for message, center_index in zip(messages, rssi_assigned_centers):
        if message.participant is None:
            message.participant = participants[center_index]


def auto_assign_participant_addresses(messages, participants):
    """

    :type messages: list of Message
    :type participants: list of Participant
    :return:
    """
    participants_without_address = [p for p in participants if not p.address_hex]

    if len(participants_without_address) == 0:
        return

    for msg in messages:
        if msg.participant in participants_without_address:
            src_address = msg.get_src_address_from_data()
            if src_address:
                participants_without_address.remove(msg.participant)
                msg.participant.address_hex = src_address



from PyQt5.QtCore import Qt, QRectF, QLineF
from PyQt5.QtGui import QPen, QColor
from PyQt5.QtWidgets import QGraphicsTextItem, QAbstractItemView

from urh import settings
from urh.simulator.GraphicsItem import GraphicsItem
from urh.simulator.SimulatorRule import SimulatorRule, SimulatorRuleCondition, ConditionType


class RuleItem(GraphicsItem):
    def __init__(self, model_item: SimulatorRule, parent=None):
        assert isinstance(model_item, SimulatorRule)
        super().__init__(model_item=model_item, parent=parent)

    @property
    def has_else_condition(self) -> bool:
        return self.model_item.has_else_condition

    def update_numbering(self):
        for child in self.get_scene_children():
            child.update_numbering()

    def update_position(self, x_pos, y_pos):
        self.setPos(x_pos - 20, y_pos)

        start_y = 5

        for child in self.get_scene_children():
            child.update_position(0, start_y)
            start_y += round(child.boundingRect().height()) - 1

        self.prepareGeometryChange()
        self.bounding_rect = self.childrenBoundingRect().adjusted(0, 0, 0, 10)

    def boundingRect(self):
        return self.bounding_rect

    def paint(self, painter, option, widget):
        pass


class RuleConditionItem(GraphicsItem):
    def __init__(self, model_item: SimulatorRuleCondition, parent=None):
        assert isinstance(model_item, SimulatorRuleCondition)
        super().__init__(model_item=model_item, parent=parent)

        self.number.setFont(self.font_bold)

        self.text = QGraphicsTextItem(self)
        self.text.setPlainText(self.model_item.type.value)
        self.text.setFont(self.font_bold)

        self.desc = QGraphicsTextItem(self)
        self.desc.setFont(self.font)

    def update_flags(self):
        if self.scene().mode == 0:
            self.set_flags(is_selectable=True, accept_hover_events=True, accept_drops=True)
        else:
            self.set_flags(is_selectable=True, accept_hover_events=True)

    def labels_width(self):
        return max(self.number.boundingRect().width() + self.text.boundingRect().width(),
                   self.desc.boundingRect().width())

    def refresh(self):
        if len(self.model_item.condition):
            if len(self.model_item.condition) > 20:
                self.desc.setPlainText(self.model_item.condition[:20] + "...")
            else:
                self.desc.setPlainText(self.model_item.condition)
        elif self.model_item.type != ConditionType.ELSE:
            self.desc.setPlainText("<Condition>")

    def update_position(self, x_pos, y_pos):
        self.setPos(x_pos, y_pos)

        start_y = 0
        start_x = ((self.scene().items_width() + 40) - (
                    self.number.boundingRect().width() + self.text.boundingRect().width())) / 2
        self.number.setPos(start_x, start_y)
        start_x += self.number.boundingRect().width()
        self.text.setPos(start_x, start_y)
        start_y += round(self.number.boundingRect().height())
        start_x = ((self.scene().items_width() + 40) - self.desc.boundingRect().width()) / 2
        self.desc.setPos(start_x, start_y)

        if self.model_item.type != ConditionType.ELSE:
            start_y += round(self.desc.boundingRect().height())

        start_y += 5

        for child in self.get_scene_children():
            child.update_position(20, start_y)
            start_y += round(child.boundingRect().height())

        width = self.scene().items_width()
        self.prepareGeometryChange()
        self.bounding_rect = QRectF(0, 0, width + 40, self.childrenBoundingRect().height() + 5)

    def update_drop_indicator(self, pos):
        rect = self.boundingRect()

        if pos.y() - rect.top() < rect.height() / 3:
            self.drop_indicator_position = QAbstractItemView.AboveItem
        elif rect.bottom() - pos.y() < rect.height() / 3:
            self.drop_indicator_position = QAbstractItemView.BelowItem
        else:
            self.drop_indicator_position = QAbstractItemView.OnItem

        self.update()

    def paint(self, painter, option, widget):
        if self.scene().mode == 1:
            self.setOpacity(1 if self.model_item.logging_active else 0.3)

        painter.setOpacity(settings.SELECTION_OPACITY)

        if self.hover_active or self.isSelected():
            painter.setBrush(settings.SELECTION_COLOR)
        elif not self.is_valid():
            painter.setBrush(QColor(255, 0, 0, 150))
        else:
            painter.setBrush(QColor.fromRgb(204, 204, 204, 255))

        height = self.number.boundingRect().height()

        if self.model_item.type != ConditionType.ELSE:
            height += self.desc.boundingRect().height()

        painter.drawRect(QRectF(0, 0, self.boundingRect().width(), height))

        painter.setBrush(Qt.NoBrush)
        painter.drawRect(self.boundingRect())

        if self.drag_over:
            self.paint_drop_indicator(painter)

    def paint_drop_indicator(self, painter):
        painter.setPen(QPen(Qt.darkRed, 2, Qt.SolidLine))
        painter.setBrush(Qt.NoBrush)
        rect = self.boundingRect()

        if self.drop_indicator_position == QAbstractItemView.AboveItem:
            painter.drawLine(QLineF(rect.topLeft(), rect.topRight()))
        elif self.drop_indicator_position == QAbstractItemView.OnItem:
            painter.drawRect(rect)
        else:
            painter.drawLine(QLineF(rect.bottomLeft(), rect.bottomRight()))



from PyQt5.QtCore import QAbstractListModel, Qt, QModelIndex
from PyQt5.QtGui import QFont

from urh import settings
from urh.plugins import Plugin


class PluginListModel(QAbstractListModel):
    def __init__(self, plugins, highlighted_plugins = None, parent=None):
        """
        :type plugins: list of Plugin
        :type highlighted_plugins: list of Plugin
        """
        super().__init__(parent)
        self.plugins = plugins
        self.highlighted_plugins = highlighted_plugins if highlighted_plugins is not None else []

    def rowCount(self, QModelIndex_parent=None, *args, **kwargs):
        return len(self.plugins)

    def data(self, index: QModelIndex, role=None):
        row = index.row()
        if role == Qt.DisplayRole:
            return self.plugins[row].name
        elif role == Qt.CheckStateRole:
            return self.plugins[row].enabled
        elif role == Qt.TextColorRole and self.plugins[row] in self.highlighted_plugins:
            return settings.HIGHLIGHT_TEXT_FOREGROUND_COLOR
        elif role == Qt.BackgroundColorRole and self.plugins[row] in self.highlighted_plugins:
            return settings.HIGHLIGHT_TEXT_BACKGROUND_COLOR
        elif role == Qt.FontRole and self.plugins[row] in self.highlighted_plugins:
            font = QFont()
            font.setBold(True)
            return font

    def setData(self, index: QModelIndex, value, role=None):
        if role == Qt.CheckStateRole:
            self.plugins[index.row()].enabled = value
        return True

    def flags(self, index):
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable


from urh import settings
from urh.ui.painting.VerticalSelection import VerticalSelection
from urh.ui.painting.ZoomableScene import ZoomableScene


class SpectrogramScene(ZoomableScene):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.removeItem(self.selection_area)

        self.selection_area = VerticalSelection(0, 0, 0, 0, fillcolor=settings.SELECTION_COLOR, opacity=0.6)
        self.selection_area.setZValue(1)
        self.addItem(self.selection_area)

    def width_spectrogram(self):
        return self.spectrogram_image.pixmap().width()



from PyQt5.QtCore import pyqtSignal
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QHeaderView
from PyQt5.QtWidgets import QMenu, QActionGroup

from urh import settings
from urh.models.SimulatorMessageTableModel import SimulatorMessageTableModel
from urh.simulator.SimulatorItem import SimulatorItem
from urh.simulator.SimulatorMessage import SimulatorMessage
from urh.ui.views.TableView import TableView


class SimulatorMessageTableView(TableView):
    open_modulator_dialog_clicked = pyqtSignal()

    def __init__(self, parent=None):
        super().__init__(parent)

        self.verticalHeader().setSectionResizeMode(QHeaderView.Fixed)
        self.horizontalHeader().setSectionResizeMode(QHeaderView.Fixed)

    def _insert_column(self, pos):
        view_type = self.model().proto_view
        index = self.model().protocol.convert_index(pos, from_view=view_type, to_view=0, decoded=False)[0]
        nbits = 1 if view_type == 0 else 4 if view_type == 1 else 8
        for row in self.selected_rows:
            msg = self.model().protocol.messages[row]
            for j in range(nbits):
                msg.insert(index + j, 0)

        self.model().update()
        self.resize_columns()

    @property
    def selected_message(self) -> SimulatorMessage:
        try:
            return self.model().protocol.messages[self.selected_rows[0]]
        except IndexError:
            return None

    def model(self) -> SimulatorMessageTableModel:
        return super().model()

    def create_context_menu(self) -> QMenu:
        menu = super().create_context_menu()

        if self.selection_is_empty:
            return menu

        menu.addSeparator()
        self._add_insert_column_menu(menu)
        menu.addSeparator()

        selected_encoding = self.selected_message.decoder

        if not all(self.model().protocol.messages[i].decoder is selected_encoding
                   for i in self.selected_rows):
            selected_encoding = None

        encoding_group = QActionGroup(self)
        encoding_menu = menu.addMenu("Enforce encoding")

        for decoding in self.model().project_manager.decodings:
            ea = encoding_menu.addAction(decoding.name)
            ea.setCheckable(True)
            ea.setActionGroup(encoding_group)

            if selected_encoding == decoding:
                ea.setChecked(True)

            ea.setData(decoding)
            ea.triggered.connect(self.on_encoding_action_triggered)

        if settings.read("multiple_modulations", False, bool):
            selected_modulation = self.model().protocol.messages[self.selected_rows[0]].modulator_index

            if not all(self.model().protocol.messages[i].modulator_index == selected_modulation
                       for i in self.selected_rows):
                selected_modulation = -1

            modulation_group = QActionGroup(self)
            modulation_menu = menu.addMenu("Modulation")

            for i, modulator in enumerate(self.model().project_manager.modulators):
                ma = modulation_menu.addAction(modulator.name)
                ma.setCheckable(True)
                ma.setActionGroup(modulation_group)

                if selected_modulation == i:
                    ma.setChecked(True)

                ma.setData(i)
                ma.triggered.connect(self.on_modulation_action_triggered)

            open_modulator_dialog_action = modulation_menu.addAction(self.tr("..."))
            open_modulator_dialog_action.triggered.connect(self.on_open_modulator_dialog_action_triggered)

        return menu

    @pyqtSlot()
    def on_encoding_action_triggered(self):
        updated_messages = []

        for row in self.selected_rows:
            self.model().protocol.messages[row].decoder = self.sender().data()
            updated_messages.append(self.model().protocol.messages[row])
        SimulatorItem.simulator_config.items_updated.emit(updated_messages)

    @pyqtSlot()
    def on_modulation_action_triggered(self):
        for row in self.selected_rows:
            self.model().protocol.messages[row].modulator_index = self.sender().data()

    @pyqtSlot()
    def on_open_modulator_dialog_action_triggered(self):
        self.open_modulator_dialog_clicked.emit()

    @pyqtSlot()
    def on_insert_column_left_action_triggered(self):
        self._insert_column(self.selection_range()[2])

    @pyqtSlot()
    def on_insert_column_right_action_triggered(self):
        self._insert_column(self.selection_range()[3])



from PyQt5.QtCore import QPoint, pyqtSignal, pyqtSlot
from PyQt5.QtWidgets import QWidget, QSizePolicy, QUndoStack, QCheckBox, QMessageBox

from urh import settings
from urh.controller.widgets.SignalFrame import SignalFrame
from urh.signalprocessing.Signal import Signal
from urh.ui.ui_tab_interpretation import Ui_Interpretation
from urh.util import util


class SignalTabController(QWidget):
    frame_closed = pyqtSignal(SignalFrame)
    not_show_again_changed = pyqtSignal()
    signal_created = pyqtSignal(int, Signal)
    files_dropped = pyqtSignal(list)
    frame_was_dropped = pyqtSignal(int, int)

    @property
    def num_frames(self):
        return len(self.signal_frames)

    @property
    def signal_frames(self):
        """

        :rtype: list of SignalFrame
        """
        splitter = self.ui.splitter
        return [splitter.widget(i) for i in range(splitter.count())
                if isinstance(splitter.widget(i), SignalFrame)]

    @property
    def signal_undo_stack(self):
        return self.undo_stack

    def __init__(self, project_manager, parent=None):
        super().__init__(parent)
        self.ui = Ui_Interpretation()
        self.ui.setupUi(self)

        util.set_splitter_stylesheet(self.ui.splitter)

        self.ui.placeholderLabel.setVisible(False)
        self.getting_started_status = None
        self.__set_getting_started_status(True)

        self.undo_stack = QUndoStack()
        self.project_manager = project_manager

        self.drag_pos = None

    def on_files_dropped(self, files):
        self.files_dropped.emit(files)

    def close_frame(self, frame:SignalFrame):
        self.frame_closed.emit(frame)

    def add_signal_frame(self, proto_analyzer, index=-1):
        self.__set_getting_started_status(False)
        sig_frame = SignalFrame(proto_analyzer, self.undo_stack, self.project_manager, parent=self)
        sframes = self.signal_frames

        if len(proto_analyzer.signal.filename) == 0:
            # new signal from "create signal from selection"
            sig_frame.ui.btnSaveSignal.show()

        self.__create_connects_for_signal_frame(signal_frame=sig_frame)
        sig_frame.signal_created.connect(self.emit_signal_created)
        sig_frame.not_show_again_changed.connect(self.not_show_again_changed.emit)
        sig_frame.ui.lineEditSignalName.setToolTip(self.tr("Sourcefile: ") + proto_analyzer.signal.filename)
        sig_frame.apply_to_all_clicked.connect(self.on_apply_to_all_clicked)

        prev_signal_frame = sframes[-1] if len(sframes) > 0 else None
        if prev_signal_frame is not None and hasattr(prev_signal_frame, "ui"):
            sig_frame.ui.cbProtoView.setCurrentIndex(prev_signal_frame.ui.cbProtoView.currentIndex())

        sig_frame.blockSignals(True)

        index = self.num_frames if index == -1 else index
        self.ui.splitter.insertWidget(index, sig_frame)
        sig_frame.blockSignals(False)

        default_view = settings.read('default_view', 0, int)
        sig_frame.ui.cbProtoView.setCurrentIndex(default_view)

        return sig_frame

    def add_empty_frame(self, filename: str, proto):
        self.__set_getting_started_status(False)
        sig_frame = SignalFrame(proto_analyzer=proto, undo_stack=self.undo_stack, project_manager=self.project_manager,
                                parent=self)

        sig_frame.ui.lineEditSignalName.setText(filename)
        self.__create_connects_for_signal_frame(signal_frame=sig_frame)

        self.ui.splitter.insertWidget(self.num_frames, sig_frame)

        return sig_frame

    def __set_getting_started_status(self, getting_started: bool):
        if getting_started == self.getting_started_status:
            return

        self.getting_started_status = getting_started
        self.ui.labelGettingStarted.setVisible(getting_started)

        if not getting_started:
            w = QWidget()
            w.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
            self.ui.splitter.addWidget(w)

    def __create_connects_for_signal_frame(self, signal_frame: SignalFrame):
        signal_frame.hold_shift = settings.read('hold_shift_to_drag', True, type=bool)
        signal_frame.drag_started.connect(self.frame_dragged)
        signal_frame.frame_dropped.connect(self.frame_dropped)
        signal_frame.files_dropped.connect(self.on_files_dropped)
        signal_frame.closed.connect(self.close_frame)

    def set_frame_numbers(self):
        for i, f in enumerate(self.signal_frames):
            f.ui.lSignalNr.setText("{0:d}:".format(i + 1))

    @pyqtSlot()
    def save_all(self):
        if self.num_frames == 0:
            return

        try:
            not_show = settings.read('not_show_save_dialog', False, type=bool)
        except TypeError:
            not_show = False

        if not not_show:
            cb = QCheckBox("Don't ask me again.")
            msg_box = QMessageBox(QMessageBox.Question, self.tr("Confirm saving all signals"),
                                  self.tr("All changed signal files will be overwritten. OK?"))
            msg_box.addButton(QMessageBox.Yes)
            msg_box.addButton(QMessageBox.No)
            msg_box.setCheckBox(cb)

            reply = msg_box.exec()
            not_show_again = cb.isChecked()
            settings.write("not_show_save_dialog", not_show_again)
            self.not_show_again_changed.emit()

            if reply != QMessageBox.Yes:
                return

        for f in self.signal_frames:
            if f.signal is None or f.signal.filename == "":
                continue
            f.signal.save()

    @pyqtSlot()
    def close_all(self):
        for f in self.signal_frames:
            f.my_close()

    @pyqtSlot(Signal)
    def on_apply_to_all_clicked(self, signal: Signal):
        for frame in self.signal_frames:
            if frame.signal is not None:
                frame.signal.noise_min_plot = signal.noise_min_plot
                frame.signal.noise_max_plot = signal.noise_max_plot

                frame.signal.block_protocol_update = True
                proto_needs_update = False

                if frame.signal.modulation_type != signal.modulation_type:
                    frame.signal.modulation_type = signal.modulation_type
                    proto_needs_update = True

                if frame.signal.center != signal.center:
                    frame.signal.center = signal.center
                    proto_needs_update = True

                if frame.signal.tolerance != signal.tolerance:
                    frame.signal.tolerance = signal.tolerance
                    proto_needs_update = True

                if frame.signal.noise_threshold != signal.noise_threshold:
                    frame.signal.noise_threshold_relative = signal.noise_threshold_relative
                    proto_needs_update = True

                if frame.signal.samples_per_symbol != signal.samples_per_symbol:
                    frame.signal.samples_per_symbol = signal.samples_per_symbol
                    proto_needs_update = True

                if frame.signal.pause_threshold != signal.pause_threshold:
                    frame.signal.pause_threshold = signal.pause_threshold
                    proto_needs_update = True

                if frame.signal.message_length_divisor != signal.message_length_divisor:
                    frame.signal.message_length_divisor = signal.message_length_divisor
                    proto_needs_update = True

                frame.signal.block_protocol_update = False

                if proto_needs_update:
                    frame.signal.protocol_needs_update.emit()

    @pyqtSlot(QPoint)
    def frame_dragged(self, pos: QPoint):
        self.drag_pos = pos

    @pyqtSlot(QPoint)
    def frame_dropped(self, pos: QPoint):
        start = self.drag_pos
        if start is None:
            return

        end = pos
        start_index = -1
        end_index = -1
        if self.num_frames > 1:
            for i, w in enumerate(self.signal_frames):
                if w.geometry().contains(start):
                    start_index = i

                if w.geometry().contains(end):
                    end_index = i

        self.swap_frames(start_index, end_index)
        self.frame_was_dropped.emit(start_index, end_index)

    @pyqtSlot(int, int)
    def swap_frames(self, from_index: int, to_index: int):
        if from_index != to_index:
            start_sig_widget = self.ui.splitter.widget(from_index)
            self.ui.splitter.insertWidget(to_index, start_sig_widget)

    @pyqtSlot()
    def on_participant_changed(self):
        for sframe in self.signal_frames:
            sframe.on_participant_changed()

    def redraw_spectrograms(self):
        for frame in self.signal_frames:
            if frame.ui.gvSpectrogram.width_spectrogram > 0:
                frame.draw_spectrogram(force_redraw=True)

    @pyqtSlot(Signal)
    def emit_signal_created(self, signal):
        try:
            index = self.signal_frames.index(self.sender()) + 1
        except ValueError:
            index = -1

        self.signal_created.emit(index, signal)
